package org.eclipse.jdt.internal.compiler.parser;

package .tmp;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.core.compiler.InvalidInputException;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.ast.Statement;
import org.eclipse.jdt.internal.compiler.parser.ConflictedParser;
import org.eclipse.jdt.internal.compiler.parser.NLSTag;
import org.eclipse.jdt.internal.compiler.parser.ScannerHelper;
import org.eclipse.jdt.internal.compiler.parser.TerminalTokens;
import org.eclipse.jdt.internal.compiler.util.Util;

public class Scanner implements TerminalTokens {
  public long sourceLevel;
  
  public long complianceLevel;
  
  public boolean useAssertAsAnIndentifier = false;
  
  public boolean containsAssertKeyword = false;
  
  public boolean useEnumAsAnIndentifier = false;
  
  public boolean recordLineSeparator = false;
  
  public char currentCharacter;
  
  public int startPosition;
  
  public int currentPosition;
  
  public int initialPosition;
  
  public int eofPosition;
  
  public boolean skipComments = false;
  
  public boolean tokenizeComments = false;
  
  public boolean tokenizeWhiteSpace = false;
  
  public char[] source;
  
  public char[] withoutUnicodeBuffer;
  
  public int withoutUnicodePtr;
  
  public boolean unicodeAsBackSlash = false;
  
  public boolean scanningFloatLiteral = false;
  
  public static final int COMMENT_ARRAYS_SIZE = 30;
  
  public int[] commentStops = new int[30];
  
  public int[] commentStarts = new int[30];
  
  public int[] commentTagStarts = new int[30];
  
  public int commentPtr = -1;
  
  protected int lastCommentLinePosition = -1;
  
  public char[][] foundTaskTags = null;
  
  public char[][] foundTaskMessages;
  
  public char[][] foundTaskPriorities = null;
  
  public int[][] foundTaskPositions;
  
  public int foundTaskCount = 0;
  
  public char[][] taskTags = null;
  
  public char[][] taskPriorities = null;
  
  public boolean isTaskCaseSensitive = true;
  
  public boolean diet = false;
  
  public int[] lineEnds = new int[250];
  
  public int linePtr = -1;
  
  public boolean wasAcr = false;
  
  public static final String END_OF_SOURCE = "End_Of_Source";
  
  public static final String INVALID_HEXA = "Invalid_Hexa_Literal";
  
  public static final String INVALID_OCTAL = "Invalid_Octal_Literal";
  
  public static final String INVALID_CHARACTER_CONSTANT = "Invalid_Character_Constant";
  
  public static final String INVALID_ESCAPE = "Invalid_Escape";
  
  public static final String INVALID_INPUT = "Invalid_Input";
  
  public static final String INVALID_UNICODE_ESCAPE = "Invalid_Unicode_Escape";
  
  public static final String INVALID_FLOAT = "Invalid_Float_Literal";
  
  public static final String INVALID_LOW_SURROGATE = "Invalid_Low_Surrogate";
  
  public static final String INVALID_HIGH_SURROGATE = "Invalid_High_Surrogate";
  
  public static final String NULL_SOURCE_STRING = "Null_Source_String";
  
  public static final String UNTERMINATED_STRING = "Unterminated_String";
  
  public static final String UNTERMINATED_COMMENT = "Unterminated_Comment";
  
  public static final String INVALID_CHAR_IN_STRING = "Invalid_Char_In_String";
  
  public static final String INVALID_DIGIT = "Invalid_Digit";
  
  private static final int[] EMPTY_LINE_ENDS = Util.EMPTY_INT_ARRAY;
  
  public static final String INVALID_BINARY = "Invalid_Binary_Literal";
  
  public static final String BINARY_LITERAL_NOT_BELOW_17 = "Binary_Literal_Not_Below_17";
  
  public static final String ILLEGAL_HEXA_LITERAL = "Illegal_Hexa_Literal";
  
  public static final String INVALID_UNDERSCORE = "Invalid_Underscore";
  
  public static final String UNDERSCORES_IN_LITERALS_NOT_BELOW_17 = "Underscores_In_Literals_Not_Below_17";
  
  static final char[] charArray_a = new char[] { 'a' };
  
  static final char[] charArray_b = new char[] { 'b' };
  
  static final char[] charArray_c = new char[] { 'c' };
  
  static final char[] charArray_d = new char[] { 'd' };
  
  static final char[] charArray_e = new char[] { 'e' };
  
  static final char[] charArray_f = new char[] { 'f' };
  
  static final char[] charArray_g = new char[] { 'g' };
  
  static final char[] charArray_h = new char[] { 'h' };
  
  static final char[] charArray_i = new char[] { 'i' };
  
  static final char[] charArray_j = new char[] { 'j' };
  
  static final char[] charArray_k = new char[] { 'k' };
  
  static final char[] charArray_l = new char[] { 'l' };
  
  static final char[] charArray_m = new char[] { 'm' };
  
  static final char[] charArray_n = new char[] { 'n' };
  
  static final char[] charArray_o = new char[] { 'o' };
  
  static final char[] charArray_p = new char[] { 'p' };
  
  static final char[] charArray_q = new char[] { 'q' };
  
  static final char[] charArray_r = new char[] { 'r' };
  
  static final char[] charArray_s = new char[] { 's' };
  
  static final char[] charArray_t = new char[] { 't' };
  
  static final char[] charArray_u = new char[] { 'u' };
  
  static final char[] charArray_v = new char[] { 'v' };
  
  static final char[] charArray_w = new char[] { 'w' };
  
  static final char[] charArray_x = new char[] { 'x' };
  
  static final char[] charArray_y = new char[] { 'y' };
  
  static final char[] charArray_z = new char[] { 'z' };
  
  static final char[] initCharArray = new char[6];
  
  static final int TableSize = 30;
  
  static final int InternalTableSize = 6;
  
  public static final int OptimizedLength = 7;
  
  public final char[][][][] charArray_length = new char[7][30][6][];
  
  public static final char[] TAG_PREFIX = "//$NON-NLS-".toCharArray();
  
  public static final int TAG_PREFIX_LENGTH = TAG_PREFIX.length;
  
  public static final char TAG_POSTFIX = '$';
  
  public static final int TAG_POSTFIX_LENGTH = 1;
  
  public static final char[] IDENTITY_COMPARISON_TAG = "//$IDENTITY-COMPARISON$".toCharArray();
  
  public boolean[] validIdentityComparisonLines;
  
  public boolean checkUninternedIdentityComparison;
  
  private NLSTag[] nlsTags = null;
  
  protected int nlsTagsPtr;
  
  public boolean checkNonExternalizedStringLiterals;
  
  protected int lastPosition;
  
  public boolean returnOnlyGreater = false;
  
  int newEntry2;
  
  int newEntry3;
  
  int newEntry4;
  
  int newEntry5;
  
  int newEntry6;
  
  public boolean insideRecovery;
  
  int[] lookBack;
  
  int nextToken;
  
  private VanguardScanner vanguardScanner;
  
  private VanguardParser vanguardParser;
  
  ConflictedParser activeParser;
  
  private boolean consumingEllipsisAnnotations;
  
  public static final int RoundBracket = 0;
  
  public static final int SquareBracket = 1;
  
  public static final int CurlyBracket = 2;
  
  public static final int BracketKinds = 3;
  
  public static final int LOW_SURROGATE_MIN_VALUE = 56320;
  
  public static final int HIGH_SURROGATE_MIN_VALUE = 55296;
  
  public static final int HIGH_SURROGATE_MAX_VALUE = 56319;
  
  public static final int LOW_SURROGATE_MAX_VALUE = 57343;
  
  public Scanner(boolean tokenizeComments, boolean tokenizeWhiteSpace, boolean checkNonExternalizedStringLiterals, long sourceLevel, long complianceLevel, char[][] taskTags, char[][] taskPriorities, boolean isTaskCaseSensitive) {
    for (int i = 0; i < 6; i++) {
      for (int j = 0; j < 30; j++) {
        for (int k = 0; k < 6; k++)
          this.charArray_length[i][j][k] = initCharArray; 
      } 
    } 
    this.newEntry2 = 0;
    this.newEntry3 = 0;
    this.newEntry4 = 0;
    this.newEntry5 = 0;
    this.newEntry6 = 0;
    this.insideRecovery = false;
    this.lookBack = new int[2];
    this.nextToken = 0;
    this.activeParser = null;
    this.consumingEllipsisAnnotations = false;
    this.eofPosition = Integer.MAX_VALUE;
    this.tokenizeComments = tokenizeComments;
    this.tokenizeWhiteSpace = tokenizeWhiteSpace;
    this.sourceLevel = sourceLevel;
    this.lookBack[1] = this.nextToken = 0;
    this.lookBack[0] = this.nextToken = 0;
    this.consumingEllipsisAnnotations = false;
    this.complianceLevel = complianceLevel;
    this.checkNonExternalizedStringLiterals = checkNonExternalizedStringLiterals;
    if (taskTags != null) {
      int taskTagsLength = taskTags.length;
      int length = taskTagsLength;
      if (taskPriorities != null) {
        int taskPrioritiesLength = taskPriorities.length;
        if (taskPrioritiesLength != taskTagsLength)
          if (taskPrioritiesLength > taskTagsLength) {
            System.arraycopy(taskPriorities, 0, taskPriorities = new char[taskTagsLength][], 0, taskTagsLength);
          } else {
            System.arraycopy(taskTags, 0, taskTags = new char[taskPrioritiesLength][], 0, taskPrioritiesLength);
            length = taskPrioritiesLength;
          }  
        int[] initialIndexes = new int[length];
        for (int j = 0; j < length; j++)
          initialIndexes[j] = j; 
        Util.reverseQuickSort(taskTags, 0, length - 1, initialIndexes);
        char[][] temp = new char[length][];
        for (int k = 0; k < length; k++)
          temp[k] = taskPriorities[initialIndexes[k]]; 
        this.taskPriorities = temp;
      } else {
        Util.reverseQuickSort(taskTags, 0, length - 1);
      } 
      this.taskTags = taskTags;
      this.isTaskCaseSensitive = isTaskCaseSensitive;
    } 
  }
  
  public Scanner() {
    this(false, false, false, 3080192L, null, null, true);
  }
  
  public Scanner(boolean tokenizeComments, boolean tokenizeWhiteSpace, boolean checkNonExternalizedStringLiterals, long sourceLevel, char[][] taskTags, char[][] taskPriorities, boolean isTaskCaseSensitive) {
    this(tokenizeComments, tokenizeWhiteSpace, checkNonExternalizedStringLiterals, sourceLevel, sourceLevel, taskTags, taskPriorities, isTaskCaseSensitive);
  }
  
  public final boolean atEnd() {
    return (this.eofPosition <= this.currentPosition);
  }
  
  public void checkTaskTag(int commentStart, int commentEnd) throws InvalidInputException {
    char[] src = this.source;
    if (this.foundTaskCount > 0 && 
      this.foundTaskPositions[this.foundTaskCount - 1][0] >= commentStart)
      return; 
    int foundTaskIndex = this.foundTaskCount;
    char previous = src[commentStart + 1];
    for (int i = commentStart + 2; i < commentEnd && i < this.eofPosition; i++) {
      char[] tag = null;
      char[] priority = null;
      if (previous != '@')
        for (int itag = 0; itag < this.taskTags.length; itag++) {
          tag = this.taskTags[itag];
          int tagLength = tag.length;
          if (tagLength != 0)
            if (!ScannerHelper.isJavaIdentifierStart(this.complianceLevel, tag[0]) || 
              !ScannerHelper.isJavaIdentifierPart(this.complianceLevel, previous)) {
              for (int t = 0; t < tagLength; ) {
                int x = i + t;
                if (x < this.eofPosition) {
                  if (x >= commentEnd)
                    // Byte code: goto -> 529 
                  char sc;
                  char tc;
                  if ((sc = src[i + t]) != (tc = tag[t]))
                    if (!this.isTaskCaseSensitive) {
                      if (ScannerHelper.toLowerCase(sc) != ScannerHelper.toLowerCase(tc))
                        // Byte code: goto -> 529 
                    } else {
                      // Byte code: goto -> 529
                    }  
                  t++;
                } 
                // Byte code: goto -> 529
              } 
              if (i + tagLength >= commentEnd || !ScannerHelper.isJavaIdentifierPart(this.complianceLevel, src[i + tagLength - 1]) || 
                !ScannerHelper.isJavaIdentifierPart(this.complianceLevel, src[i + tagLength])) {
                if (this.foundTaskTags == null) {
                  this.foundTaskTags = new char[5][];
                  this.foundTaskMessages = new char[5][];
                  this.foundTaskPriorities = new char[5][];
                  this.foundTaskPositions = new int[5][];
                } else if (this.foundTaskCount == this.foundTaskTags.length) {
                  System.arraycopy(this.foundTaskTags, 0, this.foundTaskTags = new char[this.foundTaskCount * 2][], 0, this.foundTaskCount);
                  System.arraycopy(this.foundTaskMessages, 0, this.foundTaskMessages = new char[this.foundTaskCount * 2][], 0, this.foundTaskCount);
                  System.arraycopy(this.foundTaskPriorities, 0, this.foundTaskPriorities = new char[this.foundTaskCount * 2][], 0, this.foundTaskCount);
                  System.arraycopy(this.foundTaskPositions, 0, this.foundTaskPositions = new int[this.foundTaskCount * 2][], 0, this.foundTaskCount);
                } 
                priority = (this.taskPriorities != null && itag < this.taskPriorities.length) ? 
                  this.taskPriorities[itag] : 
                  null;
                this.foundTaskTags[this.foundTaskCount] = tag;
                this.foundTaskPriorities[this.foundTaskCount] = priority;
                (new int[2])[0] = i;
                (new int[2])[1] = i + tagLength - 1;
                this.foundTaskPositions[this.foundTaskCount] = new int[2];
                this.foundTaskMessages[this.foundTaskCount] = CharOperation.NO_CHAR;
                this.foundTaskCount++;
                i += tagLength - 1;
                break;
              } 
            }  
        }  
      previous = src[i];
    } 
    boolean containsEmptyTask = false;
    int j;
    for (j = foundTaskIndex; j < this.foundTaskCount; j++) {
      int msgStart = this.foundTaskPositions[j][0] + (this.foundTaskTags[j]).length;
      int max_value = (j + 1 < this.foundTaskCount) ? (
        this.foundTaskPositions[j + 1][0] - 1) : (
        commentEnd - 1);
      if (max_value < msgStart)
        max_value = msgStart; 
      int end = -1;
      int k;
      for (k = msgStart; k < max_value; k++) {
        char c;
        if ((c = src[k]) == '\n' || c == '\r') {
          end = k - 1;
          break;
        } 
      } 
      if (end == -1) {
        for (k = max_value; k > msgStart; k--) {
          char c;
          if ((c = src[k]) == '*') {
            end = k - 1;
            break;
          } 
        } 
        if (end == -1)
          end = max_value; 
      } 
      if (msgStart == end) {
        containsEmptyTask = true;
      } else {
        while (CharOperation.isWhitespace(src[end]) && msgStart <= end)
          end--; 
        this.foundTaskPositions[j][1] = end;
        int messageLength = end - msgStart + 1;
        char[] message = new char[messageLength];
        System.arraycopy(src, msgStart, message, 0, messageLength);
        this.foundTaskMessages[j] = message;
      } 
    } 
    if (containsEmptyTask) {
      int max;
      for (j = foundTaskIndex, max = this.foundTaskCount; j < max; j++) {
        if ((this.foundTaskMessages[j]).length == 0)
          for (int k = j + 1; k < max; k++) {
            if ((this.foundTaskMessages[k]).length != 0) {
              this.foundTaskMessages[j] = this.foundTaskMessages[k];
              this.foundTaskPositions[j][1] = this.foundTaskPositions[k][1];
              break;
            } 
          }  
      } 
    } 
  }
  
  public char[] getCurrentIdentifierSource() {
    if (this.withoutUnicodePtr != 0) {
      char[] arrayOfChar = new char[this.withoutUnicodePtr];
      System.arraycopy(
          this.withoutUnicodeBuffer, 
          1, 
          arrayOfChar, 
          0, 
          this.withoutUnicodePtr);
      return arrayOfChar;
    } 
    int length = this.currentPosition - this.startPosition;
    if (length == this.eofPosition)
      return this.source; 
    switch (length) {
      case 1:
        return optimizedCurrentTokenSource1();
      case 2:
        return optimizedCurrentTokenSource2();
      case 3:
        return optimizedCurrentTokenSource3();
      case 4:
        return optimizedCurrentTokenSource4();
      case 5:
        return optimizedCurrentTokenSource5();
      case 6:
        return optimizedCurrentTokenSource6();
    } 
    char[] result = new char[length];
    System.arraycopy(this.source, this.startPosition, result, 0, length);
    return result;
  }
  
  public int getCurrentTokenEndPosition() {
    return this.currentPosition - 1;
  }
  
  public char[] getCurrentTokenSource() {
    char[] result;
    if (this.withoutUnicodePtr != 0) {
      System.arraycopy(
          this.withoutUnicodeBuffer, 
          1, 
          result = new char[this.withoutUnicodePtr], 
          0, 
          this.withoutUnicodePtr);
    } else {
      int length;
      System.arraycopy(
          this.source, 
          this.startPosition, 
          result = new char[length = this.currentPosition - this.startPosition], 
          0, 
          length);
    } 
    return result;
  }
  
  public final String getCurrentTokenString() {
    if (this.withoutUnicodePtr != 0)
      return new String(
          this.withoutUnicodeBuffer, 
          1, 
          this.withoutUnicodePtr); 
    return new String(
        this.source, 
        this.startPosition, 
        this.currentPosition - this.startPosition);
  }
  
  public char[] getCurrentTokenSourceString() {
    char[] result;
    if (this.withoutUnicodePtr != 0) {
      System.arraycopy(this.withoutUnicodeBuffer, 2, 
          
          result = new char[this.withoutUnicodePtr - 2], 0, this.withoutUnicodePtr - 2);
    } else {
      int length;
      System.arraycopy(
          this.source, 
          this.startPosition + 1, 
          result = new char[length = this.currentPosition - this.startPosition - 2], 
          0, 
          length);
    } 
    return result;
  }
  
  public final String getCurrentStringLiteral() {
    if (this.withoutUnicodePtr != 0)
      return new String(this.withoutUnicodeBuffer, 2, this.withoutUnicodePtr - 2); 
    return new String(this.source, this.startPosition + 1, this.currentPosition - this.startPosition - 2);
  }
  
  public final char[] getRawTokenSource() {
    int length = this.currentPosition - this.startPosition;
    char[] tokenSource = new char[length];
    System.arraycopy(this.source, this.startPosition, tokenSource, 0, length);
    return tokenSource;
  }
  
  public final char[] getRawTokenSourceEnd() {
    int length = this.eofPosition - this.currentPosition - 1;
    char[] sourceEnd = new char[length];
    System.arraycopy(this.source, this.currentPosition, sourceEnd, 0, length);
    return sourceEnd;
  }
  
  public int getCurrentTokenStartPosition() {
    return this.startPosition;
  }
  
  public final int getLineEnd(int lineNumber) {
    if (this.lineEnds == null || this.linePtr == -1)
      return -1; 
    if (lineNumber > this.lineEnds.length + 1)
      return -1; 
    if (lineNumber <= 0)
      return -1; 
    if (lineNumber == this.lineEnds.length + 1)
      return this.eofPosition; 
    return this.lineEnds[lineNumber - 1];
  }
  
  public final int[] getLineEnds() {
    if (this.linePtr == -1)
      return EMPTY_LINE_ENDS; 
    int[] copy;
    System.arraycopy(this.lineEnds, 0, copy = new int[this.linePtr + 1], 0, this.linePtr + 1);
    return copy;
  }
  
  public final int getLineStart(int lineNumber) {
    if (this.lineEnds == null || this.linePtr == -1)
      return -1; 
    if (lineNumber > this.lineEnds.length + 1)
      return -1; 
    if (lineNumber <= 0)
      return -1; 
    if (lineNumber == 1)
      return this.initialPosition; 
    return this.lineEnds[lineNumber - 2] + 1;
  }
  
  public final int getNextChar() {
    try {
      if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
        this.source[this.currentPosition] == 'u') {
        getNextUnicodeChar();
      } else {
        this.unicodeAsBackSlash = false;
        if (this.withoutUnicodePtr != 0)
          unicodeStore(); 
      } 
      return this.currentCharacter;
    } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
      return -1;
    } catch (InvalidInputException invalidInputException) {
      return -1;
    } 
  }
  
  public final int getNextCharWithBoundChecks() {
    if (this.currentPosition >= this.eofPosition)
      return -1; 
    this.currentCharacter = this.source[this.currentPosition++];
    if (this.currentPosition >= this.eofPosition) {
      this.unicodeAsBackSlash = false;
      if (this.withoutUnicodePtr != 0)
        unicodeStore(); 
      return this.currentCharacter;
    } 
    if (this.currentCharacter == '\\' && this.source[this.currentPosition] == 'u') {
      try {
        getNextUnicodeChar();
      } catch (InvalidInputException invalidInputException) {
        return -1;
      } 
    } else {
      this.unicodeAsBackSlash = false;
      if (this.withoutUnicodePtr != 0)
        unicodeStore(); 
    } 
    return this.currentCharacter;
  }
  
  public final boolean getNextChar(char testedChar) {
    if (this.currentPosition >= this.eofPosition) {
      this.unicodeAsBackSlash = false;
      return false;
    } 
    int temp = this.currentPosition;
    try {
      if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
        this.source[this.currentPosition] == 'u') {
        getNextUnicodeChar();
        if (this.currentCharacter != testedChar) {
          this.currentPosition = temp;
          this.withoutUnicodePtr--;
          return false;
        } 
        return true;
      } 
      if (this.currentCharacter != testedChar) {
        this.currentPosition = temp;
        return false;
      } 
      this.unicodeAsBackSlash = false;
      if (this.withoutUnicodePtr != 0)
        unicodeStore(); 
      return true;
    } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
      this.unicodeAsBackSlash = false;
      this.currentPosition = temp;
      return false;
    } catch (InvalidInputException invalidInputException) {
      this.unicodeAsBackSlash = false;
      this.currentPosition = temp;
      return false;
    } 
  }
  
  public final int getNextChar(char testedChar1, char testedChar2) {
    if (this.currentPosition >= this.eofPosition)
      return -1; 
    int temp = this.currentPosition;
    try {
      int result;
      if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
        this.source[this.currentPosition] == 'u') {
        getNextUnicodeChar();
        if (this.currentCharacter == testedChar1) {
          result = 0;
        } else if (this.currentCharacter == testedChar2) {
          result = 1;
        } else {
          this.currentPosition = temp;
          this.withoutUnicodePtr--;
          result = -1;
        } 
        return result;
      } 
      if (this.currentCharacter == testedChar1) {
        result = 0;
      } else if (this.currentCharacter == testedChar2) {
        result = 1;
      } else {
        this.currentPosition = temp;
        return -1;
      } 
      if (this.withoutUnicodePtr != 0)
        unicodeStore(); 
      return result;
    } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
      this.currentPosition = temp;
      return -1;
    } catch (InvalidInputException invalidInputException) {
      this.currentPosition = temp;
      return -1;
    } 
  }
  
  private final void consumeDigits(int radix) throws InvalidInputException {
    consumeDigits(radix, false);
  }
  
  private final void consumeDigits(int radix, boolean expectingDigitFirst) throws InvalidInputException {
    switch (consumeDigits0(radix, 1, 2, expectingDigitFirst)) {
      case 1:
        if (this.sourceLevel < 3342336L)
          throw new InvalidInputException("Underscores_In_Literals_Not_Below_17"); 
        break;
      case 2:
        if (this.sourceLevel < 3342336L)
          throw new InvalidInputException("Underscores_In_Literals_Not_Below_17"); 
        throw new InvalidInputException("Invalid_Underscore");
    } 
  }
  
  private final int consumeDigits0(int radix, int usingUnderscore, int invalidPosition, boolean expectingDigitFirst) throws InvalidInputException {
    int kind = 0;
    if (getNextChar('_')) {
      if (expectingDigitFirst)
        return invalidPosition; 
      kind = usingUnderscore;
      do {
      
      } while (getNextChar('_'));
    } 
    if (getNextCharAsDigit(radix)) {
      do {
      
      } while (getNextCharAsDigit(radix));
      int kind2 = consumeDigits0(radix, usingUnderscore, invalidPosition, false);
      if (kind2 == 0)
        return kind; 
      return kind2;
    } 
    if (kind == usingUnderscore)
      return invalidPosition; 
    return kind;
  }
  
  public final boolean getNextCharAsDigit() throws InvalidInputException {
    if (this.currentPosition >= this.eofPosition)
      return false; 
    int temp = this.currentPosition;
    try {
      if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
        this.source[this.currentPosition] == 'u') {
        getNextUnicodeChar();
        if (!ScannerHelper.isDigit(this.currentCharacter)) {
          this.currentPosition = temp;
          this.withoutUnicodePtr--;
          return false;
        } 
        return true;
      } 
      if (!ScannerHelper.isDigit(this.currentCharacter)) {
        this.currentPosition = temp;
        return false;
      } 
      if (this.withoutUnicodePtr != 0)
        unicodeStore(); 
      return true;
    } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
      this.currentPosition = temp;
      return false;
    } catch (InvalidInputException invalidInputException) {
      this.currentPosition = temp;
      return false;
    } 
  }
  
  public final boolean getNextCharAsDigit(int radix) {
    if (this.currentPosition >= this.eofPosition)
      return false; 
    int temp = this.currentPosition;
    try {
      if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
        this.source[this.currentPosition] == 'u') {
        getNextUnicodeChar();
        if (ScannerHelper.digit(this.currentCharacter, radix) == -1) {
          this.currentPosition = temp;
          this.withoutUnicodePtr--;
          return false;
        } 
        return true;
      } 
      if (ScannerHelper.digit(this.currentCharacter, radix) == -1) {
        this.currentPosition = temp;
        return false;
      } 
      if (this.withoutUnicodePtr != 0)
        unicodeStore(); 
      return true;
    } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
      this.currentPosition = temp;
      return false;
    } catch (InvalidInputException invalidInputException) {
      this.currentPosition = temp;
      return false;
    } 
  }
  
  public boolean getNextCharAsJavaIdentifierPartWithBoundCheck() {
    int pos = this.currentPosition;
    if (pos >= this.eofPosition)
      return false; 
    int temp2 = this.withoutUnicodePtr;
    try {
      boolean unicode = false;
      this.currentCharacter = this.source[this.currentPosition++];
      if (this.currentPosition < this.eofPosition && 
        this.currentCharacter == '\\' && this.source[this.currentPosition] == 'u') {
        getNextUnicodeChar();
        unicode = true;
      } 
      char c = this.currentCharacter;
      boolean isJavaIdentifierPart = false;
      if (c >= '?' && c <= '?') {
        if (this.complianceLevel < 3211264L) {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        } 
        char low = (char)getNextCharWithBoundChecks();
        if (low < '?' || low > '?') {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        } 
        isJavaIdentifierPart = ScannerHelper.isJavaIdentifierPart(this.complianceLevel, c, low);
      } else {
        if (c >= '?' && c <= '?') {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        } 
        isJavaIdentifierPart = ScannerHelper.isJavaIdentifierPart(this.complianceLevel, c);
      } 
      if (unicode) {
        if (!isJavaIdentifierPart) {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        } 
        return true;
      } 
      if (!isJavaIdentifierPart) {
        this.currentPosition = pos;
        return false;
      } 
      if (this.withoutUnicodePtr != 0)
        unicodeStore(); 
      return true;
    } catch (InvalidInputException invalidInputException) {
      this.currentPosition = pos;
      this.withoutUnicodePtr = temp2;
      return false;
    } 
  }
  
  public boolean getNextCharAsJavaIdentifierPart() {
    int pos;
    if ((pos = this.currentPosition) >= this.eofPosition)
      return false; 
    int temp2 = this.withoutUnicodePtr;
    try {
      boolean unicode = false;
      if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
        this.source[this.currentPosition] == 'u') {
        getNextUnicodeChar();
        unicode = true;
      } 
      char c = this.currentCharacter;
      boolean isJavaIdentifierPart = false;
      if (c >= '?' && c <= '?') {
        if (this.complianceLevel < 3211264L) {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        } 
        char low = (char)getNextChar();
        if (low < '?' || low > '?') {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        } 
        isJavaIdentifierPart = ScannerHelper.isJavaIdentifierPart(this.complianceLevel, c, low);
      } else {
        if (c >= '?' && c <= '?') {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        } 
        isJavaIdentifierPart = ScannerHelper.isJavaIdentifierPart(this.complianceLevel, c);
      } 
      if (unicode) {
        if (!isJavaIdentifierPart) {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        } 
        return true;
      } 
      if (!isJavaIdentifierPart) {
        this.currentPosition = pos;
        return false;
      } 
      if (this.withoutUnicodePtr != 0)
        unicodeStore(); 
      return true;
    } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
      this.currentPosition = pos;
      this.withoutUnicodePtr = temp2;
      return false;
    } catch (InvalidInputException invalidInputException) {
      this.currentPosition = pos;
      this.withoutUnicodePtr = temp2;
      return false;
    } 
  }
  
  public int scanIdentifier() throws InvalidInputException {
    int whiteStart = 0;
    this.withoutUnicodePtr = 0;
    whiteStart = this.currentPosition;
    boolean hasWhiteSpaces = false;
    boolean checkIfUnicode = false;
    while (true) {
      boolean isWhiteSpace;
      int unicodePtr = this.withoutUnicodePtr;
      int offset = this.currentPosition;
      this.startPosition = this.currentPosition;
      if (this.currentPosition < this.eofPosition) {
        this.currentCharacter = this.source[this.currentPosition++];
        checkIfUnicode = (this.currentPosition < this.eofPosition && 
          this.currentCharacter == '\\' && 
          this.source[this.currentPosition] == 'u');
      } else {
        if (this.tokenizeWhiteSpace && whiteStart != this.currentPosition - 1) {
          this.currentPosition--;
          this.startPosition = whiteStart;
          return 1000;
        } 
        return 60;
      } 
      if (checkIfUnicode) {
        isWhiteSpace = jumpOverUnicodeWhiteSpace();
        offset = this.currentPosition - offset;
      } else {
        offset = this.currentPosition - offset;
        switch (this.currentCharacter) {
          case '\t':
          case '\n':
          case '\f':
          case '\r':
          case ' ':
            isWhiteSpace = true;
            break;
          default:
            isWhiteSpace = false;
            break;
        } 
      } 
      if (isWhiteSpace)
        hasWhiteSpaces = true; 
      if (!isWhiteSpace) {
        boolean isJavaIdStart;
        if (hasWhiteSpaces) {
          if (this.tokenizeWhiteSpace) {
            this.currentPosition -= offset;
            this.startPosition = whiteStart;
            if (checkIfUnicode)
              this.withoutUnicodePtr = unicodePtr; 
            return 1000;
          } 
          if (checkIfUnicode) {
            this.withoutUnicodePtr = 0;
            unicodeStore();
          } else {
            this.withoutUnicodePtr = 0;
          } 
        } 
        char c = this.currentCharacter;
        if (c < '') {
          if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x40) != 0)
            return scanIdentifierOrKeywordWithBoundCheck(); 
          return 118;
        } 
        if (c >= '?' && c <= '?') {
          if (this.complianceLevel < 3211264L)
            throw new InvalidInputException("Invalid_Unicode_Escape"); 
          char low = (char)getNextCharWithBoundChecks();
          if (low < '?' || low > '?')
            throw new InvalidInputException("Invalid_Low_Surrogate"); 
          isJavaIdStart = ScannerHelper.isJavaIdentifierStart(this.complianceLevel, c, low);
        } else {
          if (c >= '?' && c <= '?') {
            if (this.complianceLevel < 3211264L)
              throw new InvalidInputException("Invalid_Unicode_Escape"); 
            throw new InvalidInputException("Invalid_High_Surrogate");
          } 
          isJavaIdStart = ScannerHelper.isJavaIdentifierStart(this.complianceLevel, c);
        } 
        if (isJavaIdStart)
          return scanIdentifierOrKeywordWithBoundCheck(); 
        return 118;
      } 
    } 
  }
  
  public void ungetToken(int unambiguousToken) {
    if (this.nextToken != 0)
      throw new ArrayIndexOutOfBoundsException("Single cell array overflow"); 
    this.nextToken = unambiguousToken;
  }
  
  public int getNextToken() throws InvalidInputException {
    if (this.nextToken != 0) {
      int i = this.nextToken;
      this.nextToken = 0;
      return i;
    } 
    int token = getNextToken0();
    if (this.activeParser == null)
      return token; 
    if (token == 24 || token == 11 || token == 37) {
      token = disambiguatedToken(token);
    } else if (token == 113) {
      this.consumingEllipsisAnnotations = false;
    } 
    this.lookBack[0] = this.lookBack[1];
    this.lookBack[1] = token;
    return token;
  }
  
  protected int getNextToken0() throws InvalidInputException {
    this.wasAcr = false;
    if (this.diet) {
      jumpOverMethodBody();
      this.diet = false;
      return (this.currentPosition > this.eofPosition) ? 60 : 32;
    } 
    int whiteStart = 0;
    try {
      boolean isJavaIdStart;
      while (true) {
        int temp, test, lookAhead;
        this.withoutUnicodePtr = 0;
        whiteStart = this.currentPosition;
        boolean hasWhiteSpaces = false;
        boolean checkIfUnicode = false;
        while (true) {
          boolean isWhiteSpace;
          int unicodePtr = this.withoutUnicodePtr;
          int offset = this.currentPosition;
          this.startPosition = this.currentPosition;
          try {
            checkIfUnicode = ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
              this.source[this.currentPosition] == 'u');
          } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
            if (this.tokenizeWhiteSpace && whiteStart != this.currentPosition - 1) {
              this.currentPosition--;
              this.startPosition = whiteStart;
              return 1000;
            } 
            if (this.currentPosition > this.eofPosition)
              return 60; 
          } 
          if (this.currentPosition > this.eofPosition) {
            if (this.tokenizeWhiteSpace && whiteStart != this.currentPosition - 1) {
              this.currentPosition--;
              this.startPosition = whiteStart;
              return 1000;
            } 
            return 60;
          } 
          if (checkIfUnicode) {
            isWhiteSpace = jumpOverUnicodeWhiteSpace();
            offset = this.currentPosition - offset;
          } else {
            offset = this.currentPosition - offset;
            if ((this.currentCharacter == '\r' || this.currentCharacter == '\n') && 
              this.recordLineSeparator)
              pushLineSeparator(); 
            switch (this.currentCharacter) {
              case '\t':
              case '\n':
              case '\f':
              case '\r':
              case ' ':
                isWhiteSpace = true;
                break;
              default:
                isWhiteSpace = false;
                break;
            } 
          } 
          if (isWhiteSpace)
            hasWhiteSpaces = true; 
          if (!isWhiteSpace) {
            if (hasWhiteSpaces) {
              if (this.tokenizeWhiteSpace) {
                this.currentPosition -= offset;
                this.startPosition = whiteStart;
                if (checkIfUnicode)
                  this.withoutUnicodePtr = unicodePtr; 
                return 1000;
              } 
              if (checkIfUnicode) {
                this.withoutUnicodePtr = 0;
                unicodeStore();
                break;
              } 
              this.withoutUnicodePtr = 0;
            } 
            break;
          } 
        } 
        switch (this.currentCharacter) {
          case '@':
            return 37;
          case '(':
            return 24;
          case ')':
            return 25;
          case '{':
            return 49;
          case '}':
            return 32;
          case '[':
            return 10;
          case ']':
            return 64;
          case ';':
            return 28;
          case ',':
            return 33;
          case '.':
            if (getNextCharAsDigit())
              return scanNumber(true); 
            temp = this.currentPosition;
            if (getNextChar('.')) {
              if (getNextChar('.'))
                return 113; 
              this.currentPosition = temp;
              return 3;
            } 
            this.currentPosition = temp;
            return 3;
          case '+':
            if ((test = getNextChar('+', '=')) == 0)
              return 1; 
            if (test > 0)
              return 84; 
            return 4;
          case '-':
            if ((test = getNextChar('-', '=')) == 0)
              return 2; 
            if (test > 0)
              return 85; 
            if (getNextChar('>'))
              return 110; 
            return 5;
          case '~':
            return 63;
          case '!':
            if (getNextChar('='))
              return 20; 
            return 62;
          case '*':
            if (getNextChar('='))
              return 86; 
            return 6;
          case '%':
            if (getNextChar('='))
              return 91; 
            return 8;
          case '<':
            if ((test = getNextChar('=', '<')) == 0)
              return 12; 
            if (test > 0) {
              if (getNextChar('='))
                return 92; 
              return 18;
            } 
            return 11;
          case '>':
            if (this.returnOnlyGreater)
              return 15; 
            if ((test = getNextChar('=', '>')) == 0)
              return 13; 
            if (test > 0) {
              if ((test = getNextChar('=', '>')) == 0)
                return 93; 
              if (test > 0) {
                if (getNextChar('='))
                  return 94; 
                return 16;
              } 
              return 14;
            } 
            return 15;
          case '=':
            if (getNextChar('='))
              return 19; 
            return 70;
          case '&':
            if ((test = getNextChar('&', '=')) == 0)
              return 30; 
            if (test > 0)
              return 88; 
            return 21;
          case '|':
            if ((test = getNextChar('|', '=')) == 0)
              return 31; 
            if (test > 0)
              return 89; 
            return 26;
          case '^':
            if (getNextChar('='))
              return 90; 
            return 23;
          case '?':
            return 29;
          case ':':
            if (getNextChar(':'))
              return 7; 
            return 61;
          case '\'':
            if ((test = getNextChar('\n', '\r')) == 0)
              throw new InvalidInputException("Invalid_Character_Constant"); 
            if (test > 0) {
              for (int i = 0; i < 3 && 
                this.currentPosition + i != this.eofPosition; i++) {
                if (this.source[this.currentPosition + i] == '\n')
                  break; 
                if (this.source[this.currentPosition + i] == '\'') {
                  this.currentPosition += i + 1;
                  break;
                } 
              } 
              throw new InvalidInputException("Invalid_Character_Constant");
            } 
            if (getNextChar('\'')) {
              for (int i = 0; i < 3 && 
                this.currentPosition + i != this.eofPosition; i++) {
                if (this.source[this.currentPosition + i] == '\n')
                  break; 
                if (this.source[this.currentPosition + i] == '\'') {
                  this.currentPosition += i + 1;
                  break;
                } 
              } 
              throw new InvalidInputException("Invalid_Character_Constant");
            } 
            if (getNextChar('\\')) {
              if (this.unicodeAsBackSlash) {
                this.unicodeAsBackSlash = false;
                if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && this.source[this.currentPosition] == 'u') {
                  getNextUnicodeChar();
                } else if (this.withoutUnicodePtr != 0) {
                  unicodeStore();
                } 
              } else {
                this.currentCharacter = this.source[this.currentPosition++];
              } 
              scanEscapeCharacter();
            } else {
              this.unicodeAsBackSlash = false;
              checkIfUnicode = false;
              try {
                checkIfUnicode = ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                  this.source[this.currentPosition] == 'u');
              } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
                this.currentPosition--;
                throw new InvalidInputException("Invalid_Character_Constant");
              } 
              if (checkIfUnicode) {
                getNextUnicodeChar();
              } else if (this.withoutUnicodePtr != 0) {
                unicodeStore();
              } 
            } 
            if (getNextChar('\''))
              return 47; 
            for (lookAhead = 0; lookAhead < 20 && 
              this.currentPosition + lookAhead != this.eofPosition; lookAhead++) {
              if (this.source[this.currentPosition + lookAhead] == '\n')
                break; 
              if (this.source[this.currentPosition + lookAhead] == '\'') {
                this.currentPosition += lookAhead + 1;
                break;
              } 
            } 
            throw new InvalidInputException("Invalid_Character_Constant");
          case '"':
            try {
              this.unicodeAsBackSlash = false;
              boolean isUnicode = false;
              if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                this.source[this.currentPosition] == 'u') {
                getNextUnicodeChar();
                isUnicode = true;
              } else if (this.withoutUnicodePtr != 0) {
                unicodeStore();
              } 
              while (this.currentCharacter != '"') {
                if (this.currentPosition >= this.eofPosition)
                  throw new InvalidInputException("Unterminated_String"); 
                if (this.currentCharacter == '\n' || this.currentCharacter == '\r') {
                  if (isUnicode) {
                    int start = this.currentPosition;
                    for (int i = 0; i < 50; i++) {
                      if (this.currentPosition >= this.eofPosition) {
                        this.currentPosition = start;
                        break;
                      } 
                      if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && this.source[this.currentPosition] == 'u') {
                        isUnicode = true;
                        getNextUnicodeChar();
                      } else {
                        isUnicode = false;
                      } 
                      if (!isUnicode && this.currentCharacter == '\n') {
                        this.currentPosition--;
                        break;
                      } 
                      if (this.currentCharacter == '"')
                        throw new InvalidInputException("Invalid_Char_In_String"); 
                    } 
                  } else {
                    this.currentPosition--;
                  } 
                  throw new InvalidInputException("Invalid_Char_In_String");
                } 
                if (this.currentCharacter == '\\') {
                  if (this.unicodeAsBackSlash) {
                    this.withoutUnicodePtr--;
                    this.unicodeAsBackSlash = false;
                    if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && this.source[this.currentPosition] == 'u') {
                      getNextUnicodeChar();
                      isUnicode = true;
                      this.withoutUnicodePtr--;
                    } else {
                      isUnicode = false;
                    } 
                  } else {
                    if (this.withoutUnicodePtr == 0)
                      unicodeInitializeBuffer(this.currentPosition - this.startPosition); 
                    this.withoutUnicodePtr--;
                    this.currentCharacter = this.source[this.currentPosition++];
                  } 
                  scanEscapeCharacter();
                  if (this.withoutUnicodePtr != 0)
                    unicodeStore(); 
                } 
                this.unicodeAsBackSlash = false;
                if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                  this.source[this.currentPosition] == 'u') {
                  getNextUnicodeChar();
                  isUnicode = true;
                  continue;
                } 
                isUnicode = false;
                if (this.withoutUnicodePtr != 0)
                  unicodeStore(); 
              } 
            } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
              this.currentPosition--;
              throw new InvalidInputException("Unterminated_String");
            } catch (InvalidInputException e) {
              if (e.getMessage().equals("Invalid_Escape"))
                for (int i = 0; i < 50 && 
                  this.currentPosition + i != this.eofPosition; i++) {
                  if (this.source[this.currentPosition + i] == '\n')
                    break; 
                  if (this.source[this.currentPosition + i] == '"') {
                    this.currentPosition += i + 1;
                    break;
                  } 
                }  
              throw e;
            } 
            return 48;
          case '/':
            if (!this.skipComments) {
              int i = getNextChar('/', '*');
              if (i == 0) {
                this.lastCommentLinePosition = this.currentPosition;
                try {
                  if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                    this.source[this.currentPosition] == 'u')
                    getNextUnicodeChar(); 
                  if (this.currentCharacter == '\\' && 
                    this.source[this.currentPosition] == '\\')
                    this.currentPosition++; 
                  boolean isUnicode = false;
                  while (this.currentCharacter != '\r' && this.currentCharacter != '\n') {
                    if (this.currentPosition >= this.eofPosition) {
                      this.lastCommentLinePosition = this.currentPosition;
                      this.currentPosition++;
                      throw new IndexOutOfBoundsException();
                    } 
                    this.lastCommentLinePosition = this.currentPosition;
                    isUnicode = false;
                    if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                      this.source[this.currentPosition] == 'u') {
                      getNextUnicodeChar();
                      isUnicode = true;
                    } 
                    if (this.currentCharacter == '\\' && 
                      this.source[this.currentPosition] == '\\')
                      this.currentPosition++; 
                  } 
                  if (this.currentCharacter == '\r' && 
                    this.eofPosition > this.currentPosition)
                    if (this.source[this.currentPosition] == '\n') {
                      this.currentPosition++;
                      this.currentCharacter = '\n';
                    } else if (this.source[this.currentPosition] == '\\' && 
                      this.source[this.currentPosition + 1] == 'u') {
                      getNextUnicodeChar();
                      isUnicode = true;
                    }  
                  recordComment(1001);
                  if (this.taskTags != null)
                    checkTaskTag(this.startPosition, this.currentPosition); 
                  if (this.currentCharacter == '\r' || this.currentCharacter == '\n') {
                    if ((this.checkNonExternalizedStringLiterals || this.checkUninternedIdentityComparison) && 
                      this.lastPosition < this.currentPosition)
                      parseTags(); 
                    if (this.recordLineSeparator)
                      if (isUnicode) {
                        pushUnicodeLineSeparator();
                      } else {
                        pushLineSeparator();
                      }  
                  } 
                  if (this.tokenizeComments)
                    return 1001; 
                } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
                  this.currentPosition--;
                  recordComment(1001);
                  if (this.taskTags != null)
                    checkTaskTag(this.startPosition, this.currentPosition); 
                  if ((this.checkNonExternalizedStringLiterals || this.checkUninternedIdentityComparison) && 
                    this.lastPosition < this.currentPosition)
                    parseTags(); 
                  if (this.tokenizeComments)
                    return 1001; 
                  this.currentPosition++;
                } 
                continue;
              } 
              if (i > 0) {
                try {
                  boolean isJavadoc = false, star = false;
                  boolean isUnicode = false;
                  this.unicodeAsBackSlash = false;
                  if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                    this.source[this.currentPosition] == 'u') {
                    getNextUnicodeChar();
                    isUnicode = true;
                  } else {
                    isUnicode = false;
                    if (this.withoutUnicodePtr != 0)
                      unicodeStore(); 
                  } 
                  if (this.currentCharacter == '*') {
                    isJavadoc = true;
                    star = true;
                  } 
                  if ((this.currentCharacter == '\r' || this.currentCharacter == '\n') && 
                    this.recordLineSeparator)
                    if (isUnicode) {
                      pushUnicodeLineSeparator();
                    } else {
                      pushLineSeparator();
                    }  
                  isUnicode = false;
                  int previous = this.currentPosition;
                  if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                    this.source[this.currentPosition] == 'u') {
                    getNextUnicodeChar();
                    isUnicode = true;
                  } else {
                    isUnicode = false;
                  } 
                  if (this.currentCharacter == '\\' && 
                    this.source[this.currentPosition] == '\\')
                    this.currentPosition++; 
                  if (this.currentCharacter == '/')
                    isJavadoc = false; 
                  int firstTag = 0;
                  while (this.currentCharacter != '/' || !star) {
                    if (this.currentPosition >= this.eofPosition)
                      throw new InvalidInputException("Unterminated_Comment"); 
                    if ((this.currentCharacter == '\r' || this.currentCharacter == '\n') && 
                      this.recordLineSeparator)
                      if (isUnicode) {
                        pushUnicodeLineSeparator();
                      } else {
                        pushLineSeparator();
                      }  
                    switch (this.currentCharacter) {
                      case '*':
                        star = true;
                        break;
                      case '@':
                        if (firstTag == 0 && isFirstTag())
                          firstTag = previous; 
                      default:
                        star = false;
                        break;
                    } 
                    previous = this.currentPosition;
                    if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                      this.source[this.currentPosition] == 'u') {
                      getNextUnicodeChar();
                      isUnicode = true;
                    } else {
                      isUnicode = false;
                    } 
                    if (this.currentCharacter == '\\' && 
                      this.source[this.currentPosition] == '\\')
                      this.currentPosition++; 
                  } 
                  int token = isJavadoc ? 1003 : 1002;
                  recordComment(token);
                  this.commentTagStarts[this.commentPtr] = firstTag;
                  if (this.taskTags != null)
                    checkTaskTag(this.startPosition, this.currentPosition); 
                  if (this.tokenizeComments)
                    return token; 
                } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
                  this.currentPosition--;
                  throw new InvalidInputException("Unterminated_Comment");
                } 
                continue;
              } 
            } 
            if (getNextChar('='))
              return 87; 
            return 9;
          case '\032':
            if (atEnd())
              return 60; 
            throw new InvalidInputException("Ctrl-Z");
        } 
        break;
      } 
      char c = this.currentCharacter;
      if (c < '') {
        if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x40) != 0)
          return scanIdentifierOrKeyword(); 
        if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x4) != 0)
          return scanNumber(false); 
        return 118;
      } 
      if (c >= '?' && c <= '?') {
        if (this.complianceLevel < 3211264L)
          throw new InvalidInputException("Invalid_Unicode_Escape"); 
        char low = (char)getNextChar();
        if (low < '?' || low > '?')
          throw new InvalidInputException("Invalid_Low_Surrogate"); 
        isJavaIdStart = ScannerHelper.isJavaIdentifierStart(this.complianceLevel, c, low);
      } else {
        if (c >= '?' && c <= '?') {
          if (this.complianceLevel < 3211264L)
            throw new InvalidInputException("Invalid_Unicode_Escape"); 
          throw new InvalidInputException("Invalid_High_Surrogate");
        } 
        isJavaIdStart = ScannerHelper.isJavaIdentifierStart(this.complianceLevel, c);
      } 
      if (isJavaIdStart)
        return scanIdentifierOrKeyword(); 
      if (ScannerHelper.isDigit(this.currentCharacter))
        return scanNumber(false); 
      return 118;
    } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
      if (this.tokenizeWhiteSpace && whiteStart != this.currentPosition - 1) {
        this.currentPosition--;
        this.startPosition = whiteStart;
        return 1000;
      } 
      return 60;
    } 
  }
  
  public void getNextUnicodeChar() throws InvalidInputException {
    int c1 = 0, c2 = 0, c3 = 0, c4 = 0, unicodeSize = 6;
    this.currentPosition++;
    if (this.currentPosition < this.eofPosition) {
      while (this.source[this.currentPosition] == 'u') {
        this.currentPosition++;
        if (this.currentPosition >= this.eofPosition) {
          this.currentPosition--;
          throw new InvalidInputException("Invalid_Unicode_Escape");
        } 
        unicodeSize++;
      } 
    } else {
      this.currentPosition--;
      throw new InvalidInputException("Invalid_Unicode_Escape");
    } 
    if (this.currentPosition + 4 > this.eofPosition) {
      this.currentPosition += this.eofPosition - this.currentPosition;
      throw new InvalidInputException("Invalid_Unicode_Escape");
    } 
    if ((c1 = ScannerHelper.getHexadecimalValue(this.source[this.currentPosition++])) > 15 || 
      c1 < 0 || (
      c2 = ScannerHelper.getHexadecimalValue(this.source[this.currentPosition++])) > 15 || 
      c2 < 0 || (
      c3 = ScannerHelper.getHexadecimalValue(this.source[this.currentPosition++])) > 15 || 
      c3 < 0 || (
      c4 = ScannerHelper.getHexadecimalValue(this.source[this.currentPosition++])) > 15 || 
      c4 < 0)
      throw new InvalidInputException("Invalid_Unicode_Escape"); 
    this.currentCharacter = (char)(((c1 * 16 + c2) * 16 + c3) * 16 + c4);
    if (this.withoutUnicodePtr == 0)
      unicodeInitializeBuffer(this.currentPosition - unicodeSize - this.startPosition); 
    unicodeStore();
    this.unicodeAsBackSlash = (this.currentCharacter == '\\');
  }
  
  public NLSTag[] getNLSTags() {
    int length = this.nlsTagsPtr;
    if (length != 0) {
      NLSTag[] result = new NLSTag[length];
      System.arraycopy(this.nlsTags, 0, result, 0, length);
      this.nlsTagsPtr = 0;
      return result;
    } 
    return null;
  }
  
  public boolean[] getIdentityComparisonLines() {
    boolean[] retVal = this.validIdentityComparisonLines;
    this.validIdentityComparisonLines = null;
    return retVal;
  }
  
  public char[] getSource() {
    return this.source;
  }
  
  protected boolean isFirstTag() {
    return true;
  }
  
  public final void jumpOverMethodBody() {
    this.wasAcr = false;
    int found = 1;
    try {
      while (true) {
        boolean isWhiteSpace, bool1;
        int test;
        this.withoutUnicodePtr = 0;
        do {
          this.startPosition = this.currentPosition;
          if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
            this.source[this.currentPosition] == 'u') {
            isWhiteSpace = jumpOverUnicodeWhiteSpace();
          } else {
            if (this.recordLineSeparator && (
              this.currentCharacter == '\r' || this.currentCharacter == '\n'))
              pushLineSeparator(); 
            isWhiteSpace = CharOperation.isWhitespace(this.currentCharacter);
          } 
        } while (isWhiteSpace);
        switch (this.currentCharacter) {
          case '{':
            found++;
            continue;
          case '}':
            found--;
            if (found == 0)
              return; 
            continue;
          case '\'':
            bool1 = getNextChar('\\');
            if (bool1) {
              try {
                if (this.unicodeAsBackSlash) {
                  this.unicodeAsBackSlash = false;
                  if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && this.source[this.currentPosition] == 'u') {
                    getNextUnicodeChar();
                  } else if (this.withoutUnicodePtr != 0) {
                    unicodeStore();
                  } 
                } else {
                  this.currentCharacter = this.source[this.currentPosition++];
                } 
                scanEscapeCharacter();
              } catch (InvalidInputException invalidInputException) {}
            } else {
              try {
                this.unicodeAsBackSlash = false;
                if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                  this.source[this.currentPosition] == 'u') {
                  getNextUnicodeChar();
                } else if (this.withoutUnicodePtr != 0) {
                  unicodeStore();
                } 
              } catch (InvalidInputException invalidInputException) {}
            } 
            getNextChar('\'');
            continue;
          case '"':
            try {
              try {
                this.unicodeAsBackSlash = false;
                if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                  this.source[this.currentPosition] == 'u') {
                  getNextUnicodeChar();
                } else if (this.withoutUnicodePtr != 0) {
                  unicodeStore();
                } 
              } catch (InvalidInputException invalidInputException) {}
              while (this.currentCharacter != '"') {
                if (this.currentPosition >= this.eofPosition)
                  return; 
                if (this.currentCharacter == '\r') {
                  if (this.source[this.currentPosition] == '\n')
                    this.currentPosition++; 
                  break;
                } 
                if (this.currentCharacter == '\n')
                  break; 
                if (this.currentCharacter == '\\')
                  try {
                    if (this.unicodeAsBackSlash) {
                      this.unicodeAsBackSlash = false;
                      if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && this.source[this.currentPosition] == 'u') {
                        getNextUnicodeChar();
                      } else if (this.withoutUnicodePtr != 0) {
                        unicodeStore();
                      } 
                    } else {
                      this.currentCharacter = this.source[this.currentPosition++];
                    } 
                    scanEscapeCharacter();
                  } catch (InvalidInputException invalidInputException) {} 
                try {
                  this.unicodeAsBackSlash = false;
                  if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                    this.source[this.currentPosition] == 'u') {
                    getNextUnicodeChar();
                    continue;
                  } 
                  if (this.withoutUnicodePtr != 0)
                    unicodeStore(); 
                } catch (InvalidInputException invalidInputException) {}
              } 
            } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
              return;
            } 
            continue;
          case '/':
            if ((test = getNextChar('/', '*')) == 0) {
              try {
                this.lastCommentLinePosition = this.currentPosition;
                if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                  this.source[this.currentPosition] == 'u')
                  getNextUnicodeChar(); 
                if (this.currentCharacter == '\\' && 
                  this.source[this.currentPosition] == '\\')
                  this.currentPosition++; 
                boolean isUnicode = false;
                while (this.currentCharacter != '\r' && this.currentCharacter != '\n') {
                  if (this.currentPosition >= this.eofPosition) {
                    this.lastCommentLinePosition = this.currentPosition;
                    this.currentPosition++;
                    throw new IndexOutOfBoundsException();
                  } 
                  this.lastCommentLinePosition = this.currentPosition;
                  isUnicode = false;
                  if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                    this.source[this.currentPosition] == 'u') {
                    isUnicode = true;
                    getNextUnicodeChar();
                  } 
                  if (this.currentCharacter == '\\' && 
                    this.source[this.currentPosition] == '\\')
                    this.currentPosition++; 
                } 
                if (this.currentCharacter == '\r' && 
                  this.eofPosition > this.currentPosition)
                  if (this.source[this.currentPosition] == '\n') {
                    this.currentPosition++;
                    this.currentCharacter = '\n';
                  } else if (this.source[this.currentPosition] == '\\' && 
                    this.source[this.currentPosition + 1] == 'u') {
                    isUnicode = true;
                    getNextUnicodeChar();
                  }  
                recordComment(1001);
                if (this.recordLineSeparator && (
                  this.currentCharacter == '\r' || this.currentCharacter == '\n')) {
                  if ((this.checkNonExternalizedStringLiterals || this.checkUninternedIdentityComparison) && 
                    this.lastPosition < this.currentPosition)
                    parseTags(); 
                  if (this.recordLineSeparator) {
                    if (isUnicode) {
                      pushUnicodeLineSeparator();
                      continue;
                    } 
                    pushLineSeparator();
                  } 
                } 
              } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
                this.currentPosition--;
                recordComment(1001);
                if ((this.checkNonExternalizedStringLiterals || this.checkUninternedIdentityComparison) && 
                  this.lastPosition < this.currentPosition)
                  parseTags(); 
                if (!this.tokenizeComments)
                  this.currentPosition++; 
              } 
              continue;
            } 
            if (test > 0) {
              boolean isJavadoc = false;
              try {
                boolean star = false;
                boolean isUnicode = false;
                this.unicodeAsBackSlash = false;
                if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                  this.source[this.currentPosition] == 'u') {
                  getNextUnicodeChar();
                  isUnicode = true;
                } else {
                  isUnicode = false;
                  if (this.withoutUnicodePtr != 0)
                    unicodeStore(); 
                } 
                if (this.currentCharacter == '*') {
                  isJavadoc = true;
                  star = true;
                } 
                if ((this.currentCharacter == '\r' || this.currentCharacter == '\n') && 
                  this.recordLineSeparator)
                  if (isUnicode) {
                    pushUnicodeLineSeparator();
                  } else {
                    pushLineSeparator();
                  }  
                isUnicode = false;
                int previous = this.currentPosition;
                if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                  this.source[this.currentPosition] == 'u') {
                  getNextUnicodeChar();
                  isUnicode = true;
                } else {
                  isUnicode = false;
                } 
                if (this.currentCharacter == '\\' && 
                  this.source[this.currentPosition] == '\\')
                  this.currentPosition++; 
                if (this.currentCharacter == '/')
                  isJavadoc = false; 
                int firstTag = 0;
                while (this.currentCharacter != '/' || !star) {
                  if (this.currentPosition >= this.eofPosition)
                    return; 
                  if ((this.currentCharacter == '\r' || this.currentCharacter == '\n') && 
                    this.recordLineSeparator)
                    if (isUnicode) {
                      pushUnicodeLineSeparator();
                    } else {
                      pushLineSeparator();
                    }  
                  switch (this.currentCharacter) {
                    case '*':
                      star = true;
                      break;
                    case '@':
                      if (firstTag == 0 && isFirstTag())
                        firstTag = previous; 
                    default:
                      star = false;
                      break;
                  } 
                  previous = this.currentPosition;
                  if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                    this.source[this.currentPosition] == 'u') {
                    getNextUnicodeChar();
                    isUnicode = true;
                  } else {
                    isUnicode = false;
                  } 
                  if (this.currentCharacter == '\\' && 
                    this.source[this.currentPosition] == '\\')
                    this.currentPosition++; 
                } 
                recordComment(isJavadoc ? 1003 : 1002);
                this.commentTagStarts[this.commentPtr] = firstTag;
              } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
                return;
              } 
            } 
            continue;
        } 
        try {
          boolean isJavaIdStart;
          char c = this.currentCharacter;
          if (c < '') {
            if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x40) != 0) {
              scanIdentifierOrKeyword();
              continue;
            } 
            if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x4) != 0)
              scanNumber(false); 
            continue;
          } 
          if (c >= '?' && c <= '?') {
            if (this.complianceLevel < 3211264L)
              throw new InvalidInputException("Invalid_Unicode_Escape"); 
            char low = (char)getNextChar();
            if (low < '?' || low > '?')
              continue; 
            isJavaIdStart = ScannerHelper.isJavaIdentifierStart(this.complianceLevel, c, low);
          } else {
            if (c >= '?' && c <= '?')
              continue; 
            isJavaIdStart = ScannerHelper.isJavaIdentifierStart(this.complianceLevel, c);
          } 
          if (isJavaIdStart)
            scanIdentifierOrKeyword(); 
        } catch (InvalidInputException invalidInputException) {}
      } 
    } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
    
    } catch (InvalidInputException invalidInputException) {}
  }
  
  public final boolean jumpOverUnicodeWhiteSpace() throws InvalidInputException {
    this.wasAcr = false;
    getNextUnicodeChar();
    return CharOperation.isWhitespace(this.currentCharacter);
  }
  
  final char[] optimizedCurrentTokenSource1() {
    char charOne = this.source[this.startPosition];
    switch (charOne) {
      case 'a':
        return charArray_a;
      case 'b':
        return charArray_b;
      case 'c':
        return charArray_c;
      case 'd':
        return charArray_d;
      case 'e':
        return charArray_e;
      case 'f':
        return charArray_f;
      case 'g':
        return charArray_g;
      case 'h':
        return charArray_h;
      case 'i':
        return charArray_i;
      case 'j':
        return charArray_j;
      case 'k':
        return charArray_k;
      case 'l':
        return charArray_l;
      case 'm':
        return charArray_m;
      case 'n':
        return charArray_n;
      case 'o':
        return charArray_o;
      case 'p':
        return charArray_p;
      case 'q':
        return charArray_q;
      case 'r':
        return charArray_r;
      case 's':
        return charArray_s;
      case 't':
        return charArray_t;
      case 'u':
        return charArray_u;
      case 'v':
        return charArray_v;
      case 'w':
        return charArray_w;
      case 'x':
        return charArray_x;
      case 'y':
        return charArray_y;
      case 'z':
        return charArray_z;
    } 
    return new char[] { charOne };
  }
  
  final char[] optimizedCurrentTokenSource2() {
    char[] src = this.source;
    int start = this.startPosition;
    char c0, c1;
    int hash = (((c0 = src[start]) << 6) + (c1 = src[start + 1])) % 30;
    char[][] table = this.charArray_length[0][hash];
    int i = this.newEntry2;
    while (++i < 6) {
      char[] charArray = table[i];
      if (c0 == charArray[0] && c1 == charArray[1])
        return charArray; 
    } 
    i = -1;
    int max = this.newEntry2;
    while (++i <= max) {
      char[] charArray = table[i];
      if (c0 == charArray[0] && c1 == charArray[1])
        return charArray; 
    } 
    if (++max >= 6)
      max = 0; 
    char[] r;
    System.arraycopy(src, start, r = new char[2], 0, 2);
    table[this.newEntry2 = max] = r;
    return r;
  }
  
  final char[] optimizedCurrentTokenSource3() {
    char[] src = this.source;
    int start = this.startPosition;
    char c1 = src[start + 1];
    char c0, c2;
    int hash = (((c0 = src[start]) << 6) + (c2 = src[start + 2])) % 30;
    char[][] table = this.charArray_length[1][hash];
    int i = this.newEntry3;
    while (++i < 6) {
      char[] charArray = table[i];
      if (c0 == charArray[0] && c1 == charArray[1] && c2 == charArray[2])
        return charArray; 
    } 
    i = -1;
    int max = this.newEntry3;
    while (++i <= max) {
      char[] charArray = table[i];
      if (c0 == charArray[0] && c1 == charArray[1] && c2 == charArray[2])
        return charArray; 
    } 
    if (++max >= 6)
      max = 0; 
    char[] r;
    System.arraycopy(src, start, r = new char[3], 0, 3);
    table[this.newEntry3 = max] = r;
    return r;
  }
  
  final char[] optimizedCurrentTokenSource4() {
    char[] src = this.source;
    int start = this.startPosition;
    char c1 = src[start + 1], c3 = src[start + 3];
    char c0, c2;
    int hash = (((c0 = src[start]) << 6) + (c2 = src[start + 2])) % 30;
    char[][] table = this.charArray_length[2][hash];
    int i = this.newEntry4;
    while (++i < 6) {
      char[] charArray = table[i];
      if (c0 == charArray[0] && 
        c1 == charArray[1] && 
        c2 == charArray[2] && 
        c3 == charArray[3])
        return charArray; 
    } 
    i = -1;
    int max = this.newEntry4;
    while (++i <= max) {
      char[] charArray = table[i];
      if (c0 == charArray[0] && 
        c1 == charArray[1] && 
        c2 == charArray[2] && 
        c3 == charArray[3])
        return charArray; 
    } 
    if (++max >= 6)
      max = 0; 
    char[] r;
    System.arraycopy(src, start, r = new char[4], 0, 4);
    table[this.newEntry4 = max] = r;
    return r;
  }
  
  final char[] optimizedCurrentTokenSource5() {
    char[] src = this.source;
    int start = this.startPosition;
    char c1 = src[start + 1], c3 = src[start + 3];
    char c0, c2, c4;
    int hash = (((c0 = src[start]) << 12) + ((c2 = src[start + 2]) << 6) + (c4 = src[start + 4])) % 30;
    char[][] table = this.charArray_length[3][hash];
    int i = this.newEntry5;
    while (++i < 6) {
      char[] charArray = table[i];
      if (c0 == charArray[0] && 
        c1 == charArray[1] && 
        c2 == charArray[2] && 
        c3 == charArray[3] && 
        c4 == charArray[4])
        return charArray; 
    } 
    i = -1;
    int max = this.newEntry5;
    while (++i <= max) {
      char[] charArray = table[i];
      if (c0 == charArray[0] && 
        c1 == charArray[1] && 
        c2 == charArray[2] && 
        c3 == charArray[3] && 
        c4 == charArray[4])
        return charArray; 
    } 
    if (++max >= 6)
      max = 0; 
    char[] r;
    System.arraycopy(src, start, r = new char[5], 0, 5);
    table[this.newEntry5 = max] = r;
    return r;
  }
  
  final char[] optimizedCurrentTokenSource6() {
    char[] src = this.source;
    int start = this.startPosition;
    char c1 = src[start + 1], c3 = src[start + 3], c5 = src[start + 5];
    char c0, c2, c4;
    int hash = (((c0 = src[start]) << 12) + ((c2 = src[start + 2]) << 6) + (c4 = src[start + 4])) % 30;
    char[][] table = this.charArray_length[4][hash];
    int i = this.newEntry6;
    while (++i < 6) {
      char[] charArray = table[i];
      if (c0 == charArray[0] && 
        c1 == charArray[1] && 
        c2 == charArray[2] && 
        c3 == charArray[3] && 
        c4 == charArray[4] && 
        c5 == charArray[5])
        return charArray; 
    } 
    i = -1;
    int max = this.newEntry6;
    while (++i <= max) {
      char[] charArray = table[i];
      if (c0 == charArray[0] && 
        c1 == charArray[1] && 
        c2 == charArray[2] && 
        c3 == charArray[3] && 
        c4 == charArray[4] && 
        c5 == charArray[5])
        return charArray; 
    } 
    if (++max >= 6)
      max = 0; 
    char[] r;
    System.arraycopy(src, start, r = new char[6], 0, 6);
    table[this.newEntry6 = max] = r;
    return r;
  }
  
  private void parseTags() {
    int position = 0;
    int currentStartPosition = this.startPosition;
    int currentLinePtr = this.linePtr;
    if (currentLinePtr >= 0)
      position = this.lineEnds[currentLinePtr] + 1; 
    while (ScannerHelper.isWhitespace(this.source[position]))
      position++; 
    if (currentStartPosition == position)
      return; 
    char[] s = null;
    int sourceEnd = this.currentPosition;
    int sourceStart = currentStartPosition;
    int sourceDelta = 0;
    if (this.withoutUnicodePtr != 0) {
      System.arraycopy(
          this.withoutUnicodeBuffer, 
          1, 
          s = new char[this.withoutUnicodePtr], 
          0, 
          this.withoutUnicodePtr);
      sourceEnd = this.withoutUnicodePtr;
      sourceStart = 1;
      sourceDelta = currentStartPosition;
    } else {
      s = this.source;
    } 
    int pos;
    if (this.checkNonExternalizedStringLiterals && (
      pos = CharOperation.indexOf(TAG_PREFIX, s, true, sourceStart, sourceEnd)) != -1) {
      if (this.nlsTags == null) {
        this.nlsTags = new NLSTag[10];
        this.nlsTagsPtr = 0;
      } 
      while (pos != -1) {
        int start = pos + TAG_PREFIX_LENGTH;
        int end = CharOperation.indexOf('$', s, start, sourceEnd);
        if (end != -1) {
          NLSTag currentTag = null;
          int currentLine = currentLinePtr + 1;
          try {
            currentTag = new NLSTag(pos + sourceDelta, end + sourceDelta, currentLine, extractInt(s, start, end));
          } catch (NumberFormatException numberFormatException) {
            currentTag = new NLSTag(pos + sourceDelta, end + sourceDelta, currentLine, -1);
          } 
          if (this.nlsTagsPtr == this.nlsTags.length)
            System.arraycopy(this.nlsTags, 0, this.nlsTags = new NLSTag[this.nlsTagsPtr + 10], 0, this.nlsTagsPtr); 
          this.nlsTags[this.nlsTagsPtr++] = currentTag;
        } else {
          end = start;
        } 
        pos = CharOperation.indexOf(TAG_PREFIX, s, true, end, sourceEnd);
      } 
    } 
    if (this.checkUninternedIdentityComparison && (
      pos = CharOperation.indexOf(IDENTITY_COMPARISON_TAG, s, true, sourceStart, sourceEnd)) != -1) {
      if (this.validIdentityComparisonLines == null)
        this.validIdentityComparisonLines = new boolean[0]; 
      int currentLine = currentLinePtr + 1;
      int length = this.validIdentityComparisonLines.length;
      System.arraycopy(this.validIdentityComparisonLines, 0, this.validIdentityComparisonLines = new boolean[currentLine + 1], 0, length);
      this.validIdentityComparisonLines[currentLine] = true;
    } 
  }
  
  private int extractInt(char[] array, int start, int end) {
    int value = 0;
    for (int i = start; i < end; i++) {
      char currentChar = array[i];
      int digit = 0;
      switch (currentChar) {
        case '0':
          digit = 0;
          break;
        case '1':
          digit = 1;
          break;
        case '2':
          digit = 2;
          break;
        case '3':
          digit = 3;
          break;
        case '4':
          digit = 4;
          break;
        case '5':
          digit = 5;
          break;
        case '6':
          digit = 6;
          break;
        case '7':
          digit = 7;
          break;
        case '8':
          digit = 8;
          break;
        case '9':
          digit = 9;
          break;
        default:
          throw new NumberFormatException();
      } 
      value *= 10;
      if (digit < 0)
        throw new NumberFormatException(); 
      value += digit;
    } 
    return value;
  }
  
  public final void pushLineSeparator() {
    if (this.currentCharacter == '\r') {
      int separatorPos = this.currentPosition - 1;
      if (this.linePtr >= 0 && this.lineEnds[this.linePtr] >= separatorPos)
        return; 
      int length = this.lineEnds.length;
      if (++this.linePtr >= length)
        System.arraycopy(this.lineEnds, 0, this.lineEnds = new int[length + 250], 0, length); 
      this.lineEnds[this.linePtr] = separatorPos;
      try {
        if (this.source[this.currentPosition] == '\n') {
          this.lineEnds[this.linePtr] = this.currentPosition;
          this.currentPosition++;
          this.wasAcr = false;
        } else {
          this.wasAcr = true;
        } 
      } catch (IndexOutOfBoundsException indexOutOfBoundsException) {
        this.wasAcr = true;
      } 
    } else if (this.currentCharacter == '\n') {
      if (this.wasAcr && this.lineEnds[this.linePtr] == this.currentPosition - 2) {
        this.lineEnds[this.linePtr] = this.currentPosition - 1;
      } else {
        int separatorPos = this.currentPosition - 1;
        if (this.linePtr >= 0 && this.lineEnds[this.linePtr] >= separatorPos)
          return; 
        int length = this.lineEnds.length;
        if (++this.linePtr >= length)
          System.arraycopy(this.lineEnds, 0, this.lineEnds = new int[length + 250], 0, length); 
        this.lineEnds[this.linePtr] = separatorPos;
      } 
      this.wasAcr = false;
    } 
  }
  
  public final void pushUnicodeLineSeparator() {
    if (this.currentCharacter == '\r') {
      if (this.source[this.currentPosition] == '\n') {
        this.wasAcr = false;
      } else {
        this.wasAcr = true;
      } 
    } else if (this.currentCharacter == '\n') {
      this.wasAcr = false;
    } 
  }
  
  public void recordComment(int token) {
    int commentStart = this.startPosition;
    int stopPosition = this.currentPosition;
    switch (token) {
      case 1001:
        commentStart = -this.startPosition;
        stopPosition = -this.lastCommentLinePosition;
        break;
      case 1002:
        stopPosition = -this.currentPosition;
        break;
    } 
    int length = this.commentStops.length;
    if (++this.commentPtr >= length) {
      int newLength = length + 300;
      System.arraycopy(this.commentStops, 0, this.commentStops = new int[newLength], 0, length);
      System.arraycopy(this.commentStarts, 0, this.commentStarts = new int[newLength], 0, length);
      System.arraycopy(this.commentTagStarts, 0, this.commentTagStarts = new int[newLength], 0, length);
    } 
    this.commentStops[this.commentPtr] = stopPosition;
    this.commentStarts[this.commentPtr] = commentStart;
  }
  
  public void resetTo(int begin, int end) {
    this.diet = false;
    this.initialPosition = this.startPosition = this.currentPosition = begin;
    if (this.source != null && this.source.length < end) {
      this.eofPosition = this.source.length;
    } else {
      this.eofPosition = (end < Integer.MAX_VALUE) ? (end + 1) : end;
    } 
    this.commentPtr = -1;
    this.foundTaskCount = 0;
    this.lookBack[1] = this.nextToken = 0;
    this.lookBack[0] = this.nextToken = 0;
    this.consumingEllipsisAnnotations = false;
  }
  
  protected final void scanEscapeCharacter() throws InvalidInputException {
    switch (this.currentCharacter) {
      case 'b':
        this.currentCharacter = '\b';
        return;
      case 't':
        this.currentCharacter = '\t';
        return;
      case 'n':
        this.currentCharacter = '\n';
        return;
      case 'f':
        this.currentCharacter = '\f';
        return;
      case 'r':
        this.currentCharacter = '\r';
        return;
      case '"':
        this.currentCharacter = '"';
        return;
      case '\'':
        this.currentCharacter = '\'';
        return;
      case '\\':
        this.currentCharacter = '\\';
        return;
    } 
    int number = ScannerHelper.getHexadecimalValue(this.currentCharacter);
    if (number >= 0 && number <= 7) {
      boolean zeroToThreeNot = (number > 3);
      if (ScannerHelper.isDigit(this.currentCharacter = this.source[this.currentPosition++])) {
        int digit = ScannerHelper.getHexadecimalValue(this.currentCharacter);
        if (digit >= 0 && digit <= 7) {
          number = number * 8 + digit;
          if (ScannerHelper.isDigit(this.currentCharacter = this.source[this.currentPosition++])) {
            if (zeroToThreeNot) {
              this.currentPosition--;
            } else {
              digit = ScannerHelper.getHexadecimalValue(this.currentCharacter);
              if (digit >= 0 && digit <= 7) {
                number = number * 8 + digit;
              } else {
                this.currentPosition--;
              } 
            } 
          } else {
            this.currentPosition--;
          } 
        } else {
          this.currentPosition--;
        } 
      } else {
        this.currentPosition--;
      } 
      if (number > 255)
        throw new InvalidInputException("Invalid_Escape"); 
      this.currentCharacter = (char)number;
    } else {
      throw new InvalidInputException("Invalid_Escape");
    } 
  }
  
  public int scanIdentifierOrKeywordWithBoundCheck() {
    int index, length;
    char[] data;
    this.useAssertAsAnIndentifier = false;
    this.useEnumAsAnIndentifier = false;
    char[] src = this.source;
    int srcLength = this.eofPosition;
    int pos;
    while ((pos = this.currentPosition) < srcLength) {
      char c = src[pos];
      if (c < '') {
        if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 
          0x3C) != 0) {
          if (this.withoutUnicodePtr != 0) {
            this.currentCharacter = c;
            unicodeStore();
          } 
          this.currentPosition++;
          continue;
        } 
        if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x102) != 0) {
          this.currentCharacter = c;
          break;
        } 
        do {
        
        } while (getNextCharAsJavaIdentifierPartWithBoundCheck());
        break;
      } 
      do {
      
      } while (getNextCharAsJavaIdentifierPartWithBoundCheck());
      break;
    } 
    if (this.withoutUnicodePtr == 0) {
      if ((length = this.currentPosition - this.startPosition) == 1)
        return 22; 
      data = this.source;
      index = this.startPosition;
    } else {
      if ((length = this.withoutUnicodePtr) == 1)
        return 22; 
      data = this.withoutUnicodeBuffer;
      index = 1;
    } 
    return internalScanIdentifierOrKeyword(index, length, data);
  }
  
  public int scanIdentifierOrKeyword() {
    int index, length;
    char[] data;
    this.useAssertAsAnIndentifier = false;
    this.useEnumAsAnIndentifier = false;
    char[] src = this.source;
    int srcLength = this.eofPosition;
    int pos;
    while ((pos = this.currentPosition) < srcLength) {
      char c = src[pos];
      if (c < '') {
        if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 
          0x3C) != 0) {
          if (this.withoutUnicodePtr != 0) {
            this.currentCharacter = c;
            unicodeStore();
          } 
          this.currentPosition++;
          continue;
        } 
        if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x102) != 0) {
          this.currentCharacter = c;
          break;
        } 
        do {
        
        } while (getNextCharAsJavaIdentifierPart());
        break;
      } 
      do {
      
      } while (getNextCharAsJavaIdentifierPart());
      break;
    } 
    if (this.withoutUnicodePtr == 0) {
      if ((length = this.currentPosition - this.startPosition) == 1)
        return 22; 
      data = this.source;
      index = this.startPosition;
    } else {
      if ((length = this.withoutUnicodePtr) == 1)
        return 22; 
      data = this.withoutUnicodeBuffer;
      index = 1;
    } 
    return internalScanIdentifierOrKeyword(index, length, data);
  }
  
  private int internalScanIdentifierOrKeyword(int index, int length, char[] data) {
    switch (data[index]) {
      case 'a':
        switch (length) {
          case 8:
            if (data[++index] == 'b' && 
              data[++index] == 's' && 
              data[++index] == 't' && 
              data[++index] == 'r' && 
              data[++index] == 'a' && 
              data[++index] == 'c' && 
              data[++index] == 't')
              return 51; 
            return 22;
          case 6:
            if (data[++index] == 's' && 
              data[++index] == 's' && 
              data[++index] == 'e' && 
              data[++index] == 'r' && 
              data[++index] == 't') {
              if (this.sourceLevel >= 3145728L) {
                this.containsAssertKeyword = true;
                return 72;
              } 
              this.useAssertAsAnIndentifier = true;
              return 22;
            } 
            return 22;
        } 
        return 22;
      case 'b':
        switch (length) {
          case 4:
            if (data[++index] == 'y' && data[++index] == 't' && data[++index] == 'e')
              return 98; 
            return 22;
          case 5:
            if (data[++index] == 'r' && 
              data[++index] == 'e' && 
              data[++index] == 'a' && 
              data[++index] == 'k')
              return 73; 
            return 22;
          case 7:
            if (data[++index] == 'o' && 
              data[++index] == 'o' && 
              data[++index] == 'l' && 
              data[++index] == 'e' && 
              data[++index] == 'a' && 
              data[++index] == 'n')
              return 97; 
            return 22;
        } 
        return 22;
      case 'c':
        switch (length) {
          case 4:
            if (data[++index] == 'a') {
              if (data[++index] == 's' && data[++index] == 'e')
                return 99; 
              return 22;
            } 
            if (data[index] == 'h' && data[++index] == 'a' && data[++index] == 'r')
              return 101; 
            return 22;
          case 5:
            if (data[++index] == 'a') {
              if (data[++index] == 't' && data[++index] == 'c' && data[++index] == 'h')
                return 100; 
              return 22;
            } 
            if (data[index] == 'l') {
              if (data[++index] == 'a' && 
                data[++index] == 's' && 
                data[++index] == 's')
                return 67; 
              return 22;
            } 
            if (data[index] == 'o' && 
              data[++index] == 'n' && 
              data[++index] == 's' && 
              data[++index] == 't')
              return 116; 
            return 22;
          case 8:
            if (data[++index] == 'o' && 
              data[++index] == 'n' && 
              data[++index] == 't' && 
              data[++index] == 'i' && 
              data[++index] == 'n' && 
              data[++index] == 'u' && 
              data[++index] == 'e')
              return 74; 
            return 22;
        } 
        return 22;
      case 'd':
        switch (length) {
          case 2:
            if (data[++index] == 'o')
              return 76; 
            return 22;
          case 6:
            if (data[++index] == 'o' && 
              data[++index] == 'u' && 
              data[++index] == 'b' && 
              data[++index] == 'l' && 
              data[++index] == 'e')
              return 102; 
            return 22;
          case 7:
            if (data[++index] == 'e' && 
              data[++index] == 'f' && 
              data[++index] == 'a' && 
              data[++index] == 'u' && 
              data[++index] == 'l' && 
              data[++index] == 't')
              return 75; 
            return 22;
        } 
        return 22;
      case 'e':
        switch (length) {
          case 4:
            if (data[++index] == 'l') {
              if (data[++index] == 's' && data[++index] == 'e')
                return 111; 
              return 22;
            } 
            if (data[index] == 'n' && 
              data[++index] == 'u' && 
              data[++index] == 'm') {
              if (this.sourceLevel >= 3211264L)
                return 69; 
              this.useEnumAsAnIndentifier = true;
              return 22;
            } 
            return 22;
          case 7:
            if (data[++index] == 'x' && 
              data[++index] == 't' && 
              data[++index] == 'e' && 
              data[++index] == 'n' && 
              data[++index] == 'd' && 
              data[++index] == 's')
              return 96; 
            return 22;
        } 
        return 22;
      case 'f':
        switch (length) {
          case 3:
            if (data[++index] == 'o' && data[++index] == 'r')
              return 77; 
            return 22;
          case 5:
            if (data[++index] == 'i') {
              if (data[++index] == 'n' && 
                data[++index] == 'a' && 
                data[++index] == 'l')
                return 52; 
              return 22;
            } 
            if (data[index] == 'l') {
              if (data[++index] == 'o' && 
                data[++index] == 'a' && 
                data[++index] == 't')
                return 103; 
              return 22;
            } 
            if (data[index] == 'a' && 
              data[++index] == 'l' && 
              data[++index] == 's' && 
              data[++index] == 'e')
              return 38; 
            return 22;
          case 7:
            if (data[++index] == 'i' && 
              data[++index] == 'n' && 
              data[++index] == 'a' && 
              data[++index] == 'l' && 
              data[++index] == 'l' && 
              data[++index] == 'y')
              return 109; 
            return 22;
        } 
        return 22;
      case 'g':
        if (length == 4 && 
          data[++index] == 'o' && 
          data[++index] == 't' && 
          data[++index] == 'o')
          return 117; 
        return 22;
      case 'i':
        switch (length) {
          case 2:
            if (data[++index] == 'f')
              return 78; 
            return 22;
          case 3:
            if (data[++index] == 'n' && data[++index] == 't')
              return 105; 
            return 22;
          case 6:
            if (data[++index] == 'm' && 
              data[++index] == 'p' && 
              data[++index] == 'o' && 
              data[++index] == 'r' && 
              data[++index] == 't')
              return 104; 
            return 22;
          case 9:
            if (data[++index] == 'n' && 
              data[++index] == 't' && 
              data[++index] == 'e' && 
              data[++index] == 'r' && 
              data[++index] == 'f' && 
              data[++index] == 'a' && 
              data[++index] == 'c' && 
              data[++index] == 'e')
              return 68; 
            return 22;
          case 10:
            if (data[++index] == 'm') {
              if (data[++index] == 'p' && 
                data[++index] == 'l' && 
                data[++index] == 'e' && 
                data[++index] == 'm' && 
                data[++index] == 'e' && 
                data[++index] == 'n' && 
                data[++index] == 't' && 
                data[++index] == 's')
                return 114; 
              return 22;
            } 
            if (data[index] == 'n' && 
              data[++index] == 's' && 
              data[++index] == 't' && 
              data[++index] == 'a' && 
              data[++index] == 'n' && 
              data[++index] == 'c' && 
              data[++index] == 'e' && 
              data[++index] == 'o' && 
              data[++index] == 'f')
              return 17; 
            return 22;
        } 
        return 22;
      case 'l':
        if (length == 4 && 
          data[++index] == 'o' && 
          data[++index] == 'n' && 
          data[++index] == 'g')
          return 106; 
        return 22;
      case 'n':
        switch (length) {
          case 3:
            if (data[++index] == 'e' && data[++index] == 'w')
              return 36; 
            return 22;
          case 4:
            if (data[++index] == 'u' && data[++index] == 'l' && data[++index] == 'l')
              return 39; 
            return 22;
          case 6:
            if (data[++index] == 'a' && 
              data[++index] == 't' && 
              data[++index] == 'i' && 
              data[++index] == 'v' && 
              data[++index] == 'e')
              return 53; 
            return 22;
        } 
        return 22;
      case 'p':
        switch (length) {
          case 6:
            if (data[++index] == 'u' && 
              data[++index] == 'b' && 
              data[++index] == 'l' && 
              data[++index] == 'i' && 
              data[++index] == 'c')
              return 56; 
            return 22;
          case 7:
            if (data[++index] == 'a') {
              if (data[++index] == 'c' && 
                data[++index] == 'k' && 
                data[++index] == 'a' && 
                data[++index] == 'g' && 
                data[++index] == 'e')
                return 95; 
              return 22;
            } 
            if (data[index] == 'r' && 
              data[++index] == 'i' && 
              data[++index] == 'v' && 
              data[++index] == 'a' && 
              data[++index] == 't' && 
              data[++index] == 'e')
              return 54; 
            return 22;
          case 9:
            if (data[++index] == 'r' && 
              data[++index] == 'o' && 
              data[++index] == 't' && 
              data[++index] == 'e' && 
              data[++index] == 'c' && 
              data[++index] == 't' && 
              data[++index] == 'e' && 
              data[++index] == 'd')
              return 55; 
            return 22;
        } 
        return 22;
      case 'r':
        if (length == 6 && 
          data[++index] == 'e' && 
          data[++index] == 't' && 
          data[++index] == 'u' && 
          data[++index] == 'r' && 
          data[++index] == 'n')
          return 79; 
        return 22;
      case 's':
        switch (length) {
          case 5:
            if (data[++index] == 'h') {
              if (data[++index] == 'o' && data[++index] == 'r' && data[++index] == 't')
                return 107; 
              return 22;
            } 
            if (data[index] == 'u' && 
              data[++index] == 'p' && 
              data[++index] == 'e' && 
              data[++index] == 'r')
              return 34; 
            return 22;
          case 6:
            if (data[++index] == 't') {
              if (data[++index] == 'a' && 
                data[++index] == 't' && 
                data[++index] == 'i' && 
                data[++index] == 'c')
                return 40; 
              return 22;
            } 
            if (data[index] == 'w' && 
              data[++index] == 'i' && 
              data[++index] == 't' && 
              data[++index] == 'c' && 
              data[++index] == 'h')
              return 80; 
            return 22;
          case 8:
            if (data[++index] == 't' && 
              data[++index] == 'r' && 
              data[++index] == 'i' && 
              data[++index] == 'c' && 
              data[++index] == 't' && 
              data[++index] == 'f' && 
              data[++index] == 'p')
              return 57; 
            return 22;
          case 12:
            if (data[++index] == 'y' && 
              data[++index] == 'n' && 
              data[++index] == 'c' && 
              data[++index] == 'h' && 
              data[++index] == 'r' && 
              data[++index] == 'o' && 
              data[++index] == 'n' && 
              data[++index] == 'i' && 
              data[++index] == 'z' && 
              data[++index] == 'e' && 
              data[++index] == 'd')
              return 41; 
            return 22;
        } 
        return 22;
      case 't':
        switch (length) {
          case 3:
            if (data[++index] == 'r' && data[++index] == 'y')
              return 82; 
            return 22;
          case 4:
            if (data[++index] == 'h') {
              if (data[++index] == 'i' && data[++index] == 's')
                return 35; 
              return 22;
            } 
            if (data[index] == 'r' && data[++index] == 'u' && data[++index] == 'e')
              return 42; 
            return 22;
          case 5:
            if (data[++index] == 'h' && 
              data[++index] == 'r' && 
              data[++index] == 'o' && 
              data[++index] == 'w')
              return 81; 
            return 22;
          case 6:
            if (data[++index] == 'h' && 
              data[++index] == 'r' && 
              data[++index] == 'o' && 
              data[++index] == 'w' && 
              data[++index] == 's')
              return 112; 
            return 22;
          case 9:
            if (data[++index] == 'r' && 
              data[++index] == 'a' && 
              data[++index] == 'n' && 
              data[++index] == 's' && 
              data[++index] == 'i' && 
              data[++index] == 'e' && 
              data[++index] == 'n' && 
              data[++index] == 't')
              return 58; 
            return 22;
        } 
        return 22;
      case 'v':
        switch (length) {
          case 4:
            if (data[++index] == 'o' && data[++index] == 'i' && data[++index] == 'd')
              return 108; 
            return 22;
          case 8:
            if (data[++index] == 'o' && 
              data[++index] == 'l' && 
              data[++index] == 'a' && 
              data[++index] == 't' && 
              data[++index] == 'i' && 
              data[++index] == 'l' && 
              data[++index] == 'e')
              return 59; 
            return 22;
        } 
        return 22;
      case 'w':
        switch (length) {
          case 5:
            if (data[++index] == 'h' && 
              data[++index] == 'i' && 
              data[++index] == 'l' && 
              data[++index] == 'e')
              return 71; 
            return 22;
        } 
        return 22;
    } 
    return 22;
  }
  
  public int scanNumber(boolean dotPrefix) throws InvalidInputException {
    boolean floating = dotPrefix;
    if (!dotPrefix && this.currentCharacter == '0') {
      if (getNextChar('x', 'X') >= 0) {
        int start = this.currentPosition;
        consumeDigits(16, true);
        int end = this.currentPosition;
        if (getNextChar('l', 'L') >= 0) {
          if (end == start)
            throw new InvalidInputException("Invalid_Hexa_Literal"); 
          return 44;
        } 
        if (getNextChar('.')) {
          boolean hasNoDigitsBeforeDot = (end == start);
          start = this.currentPosition;
          consumeDigits(16, true);
          end = this.currentPosition;
          if (hasNoDigitsBeforeDot && end == start) {
            if (this.sourceLevel < 3211264L)
              throw new InvalidInputException("Illegal_Hexa_Literal"); 
            throw new InvalidInputException("Invalid_Hexa_Literal");
          } 
          if (getNextChar('p', 'P') >= 0) {
            this.unicodeAsBackSlash = false;
            if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
              this.source[this.currentPosition] == 'u') {
              getNextUnicodeChar();
            } else if (this.withoutUnicodePtr != 0) {
              unicodeStore();
            } 
            if (this.currentCharacter == '-' || 
              this.currentCharacter == '+') {
              this.unicodeAsBackSlash = false;
              if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
                this.source[this.currentPosition] == 'u') {
                getNextUnicodeChar();
              } else if (this.withoutUnicodePtr != 0) {
                unicodeStore();
              } 
            } 
            if (!ScannerHelper.isDigit(this.currentCharacter)) {
              if (this.sourceLevel < 3211264L)
                throw new InvalidInputException("Illegal_Hexa_Literal"); 
              if (this.currentCharacter == '_') {
                consumeDigits(10);
                throw new InvalidInputException("Invalid_Underscore");
              } 
              throw new InvalidInputException("Invalid_Hexa_Literal");
            } 
            consumeDigits(10);
            if (getNextChar('f', 'F') >= 0) {
              if (this.sourceLevel < 3211264L)
                throw new InvalidInputException("Illegal_Hexa_Literal"); 
              return 45;
            } 
            if (getNextChar('d', 'D') >= 0) {
              if (this.sourceLevel < 3211264L)
                throw new InvalidInputException("Illegal_Hexa_Literal"); 
              return 46;
            } 
            if (getNextChar('l', 'L') >= 0) {
              if (this.sourceLevel < 3211264L)
                throw new InvalidInputException("Illegal_Hexa_Literal"); 
              throw new InvalidInputException("Invalid_Hexa_Literal");
            } 
            if (this.sourceLevel < 3211264L)
              throw new InvalidInputException("Illegal_Hexa_Literal"); 
            return 46;
          } 
          if (this.sourceLevel < 3211264L)
            throw new InvalidInputException("Illegal_Hexa_Literal"); 
          throw new InvalidInputException("Invalid_Hexa_Literal");
        } 
        if (getNextChar('p', 'P') >= 0) {
          if (end == start) {
            if (this.sourceLevel < 3211264L)
              throw new InvalidInputException("Illegal_Hexa_Literal"); 
            throw new InvalidInputException("Invalid_Hexa_Literal");
          } 
          this.unicodeAsBackSlash = false;
          if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
            this.source[this.currentPosition] == 'u') {
            getNextUnicodeChar();
          } else if (this.withoutUnicodePtr != 0) {
            unicodeStore();
          } 
          if (this.currentCharacter == '-' || 
            this.currentCharacter == '+') {
            this.unicodeAsBackSlash = false;
            if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
              this.source[this.currentPosition] == 'u') {
              getNextUnicodeChar();
            } else if (this.withoutUnicodePtr != 0) {
              unicodeStore();
            } 
          } 
          if (!ScannerHelper.isDigit(this.currentCharacter)) {
            if (this.sourceLevel < 3211264L)
              throw new InvalidInputException("Illegal_Hexa_Literal"); 
            if (this.currentCharacter == '_') {
              consumeDigits(10);
              throw new InvalidInputException("Invalid_Underscore");
            } 
            throw new InvalidInputException("Invalid_Float_Literal");
          } 
          consumeDigits(10);
          if (getNextChar('f', 'F') >= 0) {
            if (this.sourceLevel < 3211264L)
              throw new InvalidInputException("Illegal_Hexa_Literal"); 
            return 45;
          } 
          if (getNextChar('d', 'D') >= 0) {
            if (this.sourceLevel < 3211264L)
              throw new InvalidInputException("Illegal_Hexa_Literal"); 
            return 46;
          } 
          if (getNextChar('l', 'L') >= 0) {
            if (this.sourceLevel < 3211264L)
              throw new InvalidInputException("Illegal_Hexa_Literal"); 
            throw new InvalidInputException("Invalid_Hexa_Literal");
          } 
          if (this.sourceLevel < 3211264L)
            throw new InvalidInputException("Illegal_Hexa_Literal"); 
          return 46;
        } 
        if (end == start)
          throw new InvalidInputException("Invalid_Hexa_Literal"); 
        return 43;
      } 
      if (getNextChar('b', 'B') >= 0) {
        int start = this.currentPosition;
        consumeDigits(2, true);
        int end = this.currentPosition;
        if (end == start) {
          if (this.sourceLevel < 3342336L)
            throw new InvalidInputException("Binary_Literal_Not_Below_17"); 
          throw new InvalidInputException("Invalid_Binary_Literal");
        } 
        if (getNextChar('l', 'L') >= 0) {
          if (this.sourceLevel < 3342336L)
            throw new InvalidInputException("Binary_Literal_Not_Below_17"); 
          return 44;
        } 
        if (this.sourceLevel < 3342336L)
          throw new InvalidInputException("Binary_Literal_Not_Below_17"); 
        return 43;
      } 
      if (getNextCharAsDigit()) {
        consumeDigits(10);
        if (getNextChar('l', 'L') >= 0)
          return 44; 
        if (getNextChar('f', 'F') >= 0)
          return 45; 
        if (getNextChar('d', 'D') >= 0)
          return 46; 
        boolean isInteger = true;
        if (getNextChar('.')) {
          isInteger = false;
          consumeDigits(10);
        } 
        if (getNextChar('e', 'E') >= 0) {
          isInteger = false;
          this.unicodeAsBackSlash = false;
          if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
            this.source[this.currentPosition] == 'u') {
            getNextUnicodeChar();
          } else if (this.withoutUnicodePtr != 0) {
            unicodeStore();
          } 
          if (this.currentCharacter == '-' || 
            this.currentCharacter == '+') {
            this.unicodeAsBackSlash = false;
            if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
              this.source[this.currentPosition] == 'u') {
              getNextUnicodeChar();
            } else if (this.withoutUnicodePtr != 0) {
              unicodeStore();
            } 
          } 
          if (!ScannerHelper.isDigit(this.currentCharacter)) {
            if (this.currentCharacter == '_') {
              consumeDigits(10);
              throw new InvalidInputException("Invalid_Underscore");
            } 
            throw new InvalidInputException("Invalid_Float_Literal");
          } 
          consumeDigits(10);
        } 
        if (getNextChar('f', 'F') >= 0)
          return 45; 
        if (getNextChar('d', 'D') >= 0 || !isInteger)
          return 46; 
        return 43;
      } 
    } 
    consumeDigits(10);
    if (!dotPrefix && getNextChar('l', 'L') >= 0)
      return 44; 
    if (!dotPrefix && getNextChar('.')) {
      consumeDigits(10, true);
      floating = true;
    } 
    if (getNextChar('e', 'E') >= 0) {
      floating = true;
      this.unicodeAsBackSlash = false;
      if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
        this.source[this.currentPosition] == 'u') {
        getNextUnicodeChar();
      } else if (this.withoutUnicodePtr != 0) {
        unicodeStore();
      } 
      if (this.currentCharacter == '-' || 
        this.currentCharacter == '+') {
        this.unicodeAsBackSlash = false;
        if ((this.currentCharacter = this.source[this.currentPosition++]) == '\\' && 
          this.source[this.currentPosition] == 'u') {
          getNextUnicodeChar();
        } else if (this.withoutUnicodePtr != 0) {
          unicodeStore();
        } 
      } 
      if (!ScannerHelper.isDigit(this.currentCharacter)) {
        if (this.currentCharacter == '_') {
          consumeDigits(10);
          throw new InvalidInputException("Invalid_Underscore");
        } 
        throw new InvalidInputException("Invalid_Float_Literal");
      } 
      consumeDigits(10);
    } 
    if (getNextChar('d', 'D') >= 0)
      return 46; 
    if (getNextChar('f', 'F') >= 0)
      return 45; 
    return floating ? 46 : 43;
  }
  
  public final int getLineNumber(int position) {
    return Util.getLineNumber(position, this.lineEnds, 0, this.linePtr);
  }
  
  public final void setSource(char[] sourceString) {
    int sourceLength;
    if (sourceString == null) {
      this.source = CharOperation.NO_CHAR;
      sourceLength = 0;
    } else {
      this.source = sourceString;
      sourceLength = sourceString.length;
    } 
    this.startPosition = -1;
    this.eofPosition = sourceLength;
    this.initialPosition = this.currentPosition = 0;
    this.containsAssertKeyword = false;
    this.linePtr = -1;
  }
  
  public final void setSource(char[] contents, CompilationResult compilationResult) {
    if (contents == null) {
      char[] cuContents = compilationResult.compilationUnit.getContents();
      setSource(cuContents);
    } else {
      setSource(contents);
    } 
    int[] lineSeparatorPositions = compilationResult.lineSeparatorPositions;
    if (lineSeparatorPositions != null) {
      this.lineEnds = lineSeparatorPositions;
      this.linePtr = lineSeparatorPositions.length - 1;
    } 
  }
  
  public final void setSource(CompilationResult compilationResult) {
    setSource(null, compilationResult);
  }
  
  public String toString() {
    if (this.startPosition == this.eofPosition)
      return "EOF\n\n" + new String(this.source); 
    if (this.currentPosition > this.eofPosition)
      return "behind the EOF\n\n" + new String(this.source); 
    if (this.currentPosition <= 0)
      return "NOT started!\n\n" + ((this.source != null) ? new String(this.source) : ""); 
    StringBuffer buffer = new StringBuffer();
    if (this.startPosition < 1000) {
      buffer.append(this.source, 0, this.startPosition);
    } else {
      buffer.append("<source beginning>\n...\n");
      int line = Util.getLineNumber(this.startPosition - 1000, this.lineEnds, 0, this.linePtr);
      int lineStart = getLineStart(line);
      buffer.append(this.source, lineStart, this.startPosition - lineStart);
    } 
    buffer.append("\n===============================\nStarts here -->");
    int middleLength = this.currentPosition - 1 - this.startPosition + 1;
    if (middleLength > -1)
      buffer.append(this.source, this.startPosition, middleLength); 
    if (this.nextToken != 0) {
      buffer.append("<-- Ends here [in pipeline " + toStringAction(this.nextToken) + "]\n===============================\n");
    } else {
      buffer.append("<-- Ends here\n===============================\n");
    } 
    buffer.append(this.source, this.currentPosition - 1 + 1, this.eofPosition - this.currentPosition - 1 - 1);
    return buffer.toString();
  }
  
  public String toStringAction(int act) {
    switch (act) {
      case 22:
        return "Identifier(" + new String(getCurrentTokenSource()) + ")";
      case 51:
        return "abstract";
      case 97:
        return "boolean";
      case 73:
        return "break";
      case 98:
        return "byte";
      case 99:
        return "case";
      case 100:
        return "catch";
      case 101:
        return "char";
      case 67:
        return "class";
      case 74:
        return "continue";
      case 75:
        return "default";
      case 76:
        return "do";
      case 102:
        return "double";
      case 111:
        return "else";
      case 96:
        return "extends";
      case 38:
        return "false";
      case 52:
        return "final";
      case 109:
        return "finally";
      case 103:
        return "float";
      case 77:
        return "for";
      case 78:
        return "if";
      case 114:
        return "implements";
      case 104:
        return "import";
      case 17:
        return "instanceof";
      case 105:
        return "int";
      case 68:
        return "interface";
      case 106:
        return "long";
      case 53:
        return "native";
      case 36:
        return "new";
      case 39:
        return "null";
      case 95:
        return "package";
      case 54:
        return "private";
      case 55:
        return "protected";
      case 56:
        return "public";
      case 79:
        return "return";
      case 107:
        return "short";
      case 40:
        return "static";
      case 34:
        return "super";
      case 80:
        return "switch";
      case 41:
        return "synchronized";
      case 35:
        return "this";
      case 81:
        return "throw";
      case 112:
        return "throws";
      case 58:
        return "transient";
      case 42:
        return "true";
      case 82:
        return "try";
      case 108:
        return "void";
      case 59:
        return "volatile";
      case 71:
        return "while";
      case 43:
        return "Integer(" + new String(getCurrentTokenSource()) + ")";
      case 44:
        return "Long(" + new String(getCurrentTokenSource()) + ")";
      case 45:
        return "Float(" + new String(getCurrentTokenSource()) + ")";
      case 46:
        return "Double(" + new String(getCurrentTokenSource()) + ")";
      case 47:
        return "Char(" + new String(getCurrentTokenSource()) + ")";
      case 48:
        return "String(" + new String(getCurrentTokenSource()) + ")";
      case 1:
        return "++";
      case 2:
        return "--";
      case 19:
        return "==";
      case 12:
        return "<=";
      case 13:
        return ">=";
      case 20:
        return "!=";
      case 18:
        return "<<";
      case 14:
        return ">>";
      case 16:
        return ">>>";
      case 84:
        return "+=";
      case 85:
        return "-=";
      case 110:
        return "->";
      case 86:
        return "*=";
      case 87:
        return "/=";
      case 88:
        return "&=";
      case 89:
        return "|=";
      case 90:
        return "^=";
      case 91:
        return "%=";
      case 92:
        return "<<=";
      case 93:
        return ">>=";
      case 94:
        return ">>>=";
      case 31:
        return "||";
      case 30:
        return "&&";
      case 4:
        return "+";
      case 5:
        return "-";
      case 62:
        return "!";
      case 8:
        return "%";
      case 23:
        return "^";
      case 21:
        return "&";
      case 6:
        return "*";
      case 26:
        return "|";
      case 63:
        return "~";
      case 9:
        return "/";
      case 15:
        return ">";
      case 11:
        return "<";
      case 24:
        return "(";
      case 25:
        return ")";
      case 49:
        return "{";
      case 32:
        return "}";
      case 10:
        return "[";
      case 64:
        return "]";
      case 28:
        return ";";
      case 29:
        return "?";
      case 61:
        return ":";
      case 7:
        return "::";
      case 33:
        return ",";
      case 3:
        return ".";
      case 70:
        return "=";
      case 60:
        return "EOF";
      case 1000:
        return "white_space(" + new String(getCurrentTokenSource()) + ")";
    } 
    return "not-a-token";
  }
  
  public void unicodeInitializeBuffer(int length) {
    this.withoutUnicodePtr = length;
    if (this.withoutUnicodeBuffer == null)
      this.withoutUnicodeBuffer = new char[length + 11]; 
    int bLength = this.withoutUnicodeBuffer.length;
    if (1 + length >= bLength)
      System.arraycopy(this.withoutUnicodeBuffer, 0, this.withoutUnicodeBuffer = new char[length + 11], 0, bLength); 
    System.arraycopy(this.source, this.startPosition, this.withoutUnicodeBuffer, 1, length);
  }
  
  public void unicodeStore() {
    int pos = ++this.withoutUnicodePtr;
    if (this.withoutUnicodeBuffer == null)
      this.withoutUnicodeBuffer = new char[10]; 
    int length = this.withoutUnicodeBuffer.length;
    if (pos == length)
      System.arraycopy(this.withoutUnicodeBuffer, 0, this.withoutUnicodeBuffer = new char[length * 2], 0, length); 
    this.withoutUnicodeBuffer[pos] = this.currentCharacter;
  }
  
  public void unicodeStore(char character) {
    int pos = ++this.withoutUnicodePtr;
    if (this.withoutUnicodeBuffer == null)
      this.withoutUnicodeBuffer = new char[10]; 
    int length = this.withoutUnicodeBuffer.length;
    if (pos == length)
      System.arraycopy(this.withoutUnicodeBuffer, 0, this.withoutUnicodeBuffer = new char[length * 2], 0, length); 
    this.withoutUnicodeBuffer[pos] = character;
  }
  
  public static boolean isIdentifier(int token) {
    return (token == 22);
  }
  
  public static boolean isLiteral(int token) {
    switch (token) {
      case 43:
      case 44:
      case 45:
      case 46:
      case 47:
      case 48:
        return true;
    } 
    return false;
  }
  
  public static boolean isKeyword(int token) {
    switch (token) {
      case 17:
      case 34:
      case 35:
      case 36:
      case 38:
      case 39:
      case 40:
      case 41:
      case 42:
      case 51:
      case 52:
      case 53:
      case 54:
      case 55:
      case 56:
      case 57:
      case 58:
      case 59:
      case 67:
      case 68:
      case 71:
      case 72:
      case 73:
      case 74:
      case 75:
      case 76:
      case 77:
      case 78:
      case 79:
      case 80:
      case 81:
      case 82:
      case 95:
      case 96:
      case 97:
      case 98:
      case 99:
      case 100:
      case 101:
      case 102:
      case 103:
      case 104:
      case 105:
      case 106:
      case 107:
      case 108:
      case 109:
      case 111:
      case 112:
      case 114:
        return true;
    } 
    return false;
  }
  
  private VanguardParser getVanguardParser() {
    if (this.vanguardParser == null) {
      this.vanguardScanner = new VanguardScanner(this.sourceLevel, this.complianceLevel);
      this.vanguardParser = new VanguardParser(this.vanguardScanner);
      this.vanguardScanner.setActiveParser((ConflictedParser)this.vanguardParser);
    } 
    this.vanguardScanner.setSource(this.source);
    this.vanguardScanner.resetTo(this.startPosition, this.eofPosition - 1);
    return this.vanguardParser;
  }
  
  protected final boolean maybeAtLambdaOrCast() {
    switch (this.lookBack[1]) {
      case 22:
      case 34:
      case 35:
      case 41:
      case 71:
      case 77:
      case 78:
      case 80:
      case 82:
      case 100:
        return false;
    } 
    return this.activeParser.atConflictScenario(24);
  }
  
  protected final boolean maybeAtReferenceExpression() {
    switch (this.lookBack[1]) {
      case 22:
        switch (this.lookBack[0]) {
          case 11:
          case 14:
          case 15:
          case 17:
          case 21:
          case 28:
          case 32:
          case 34:
          case 36:
          case 37:
          case 40:
          case 51:
          case 52:
          case 54:
          case 55:
          case 56:
          case 67:
          case 68:
          case 69:
          case 96:
          case 112:
          case 114:
            return false;
        } 
      case 0:
        return this.activeParser.atConflictScenario(11);
    } 
    return false;
  }
  
  private final boolean maybeAtEllipsisAnnotationsStart() {
    if (this.consumingEllipsisAnnotations)
      return false; 
    switch (this.lookBack[1]) {
      case 3:
      case 11:
      case 17:
      case 21:
      case 33:
      case 34:
      case 36:
      case 49:
      case 96:
      case 112:
      case 114:
        return false;
    } 
    return true;
  }
  
  protected final boolean atTypeAnnotation() {
    return !this.activeParser.atConflictScenario(37);
  }
  
  public void setActiveParser(ConflictedParser parser) {
    this.activeParser = parser;
    this.lookBack[1] = 0;
    this.lookBack[0] = 0;
  }
  
  int disambiguatedToken(int token) {
    VanguardParser parser = getVanguardParser();
    if (token == 24 && maybeAtLambdaOrCast()) {
      if (parser.parse(Goal.LambdaParameterListGoal)) {
        this.nextToken = 24;
        return 50;
      } 
      this.vanguardScanner.resetTo(this.startPosition, this.eofPosition - 1);
      if (parser.parse(Goal.IntersectionCastGoal)) {
        this.nextToken = 24;
        return 65;
      } 
    } else if (token == 11 && maybeAtReferenceExpression()) {
      if (parser.parse(Goal.ReferenceExpressionGoal)) {
        this.nextToken = 11;
        return 83;
      } 
    } else if (token == 37 && atTypeAnnotation()) {
      token = 27;
      if (maybeAtEllipsisAnnotationsStart() && 
        parser.parse(Goal.VarargTypeAnnotationGoal)) {
        this.consumingEllipsisAnnotations = true;
        this.nextToken = 27;
        return 115;
      } 
    } 
    return token;
  }
  
  protected boolean isAtAssistIdentifier() {
    return false;
  }
  
  public int fastForward(Statement unused) {
    int token;
    while (true) {
      try {
        token = getNextToken();
      } catch (InvalidInputException invalidInputException) {
        return 60;
      } 
      switch (token) {
        case 22:
          if (isAtAssistIdentifier())
            return token; 
        case 1:
        case 2:
        case 11:
        case 24:
        case 27:
        case 34:
        case 35:
        case 36:
        case 37:
        case 38:
        case 39:
        case 40:
        case 41:
        case 42:
        case 43:
        case 44:
        case 45:
        case 46:
        case 47:
        case 48:
        case 49:
        case 50:
        case 51:
        case 52:
        case 53:
        case 54:
        case 55:
        case 56:
        case 57:
        case 58:
        case 59:
        case 67:
        case 68:
        case 69:
        case 71:
        case 72:
        case 73:
        case 74:
        case 75:
        case 76:
        case 77:
        case 78:
        case 79:
        case 80:
        case 81:
        case 82:
        case 97:
        case 98:
        case 99:
        case 101:
        case 102:
        case 103:
        case 105:
        case 106:
        case 107:
        case 108:
          if (getVanguardParser().parse(Goal.BlockStatementoptGoal))
            return token; 
        case 28:
        case 60:
          return token;
        case 32:
          break;
      } 
    } 
    ungetToken(token);
    return 28;
  }
}
